# -*- coding: utf-8 -*-

import sys, os
import wx
import locale
from time import sleep
# core
from core import subprocess as mysubprocess

# PFC modules
from pfc	  import prefs
from pfc.pfc  import *
from pfc.meta import filterSpeakers
from pfc.standard import compileText, compileSchwa, compileLiaison, compileRLiaison, NumberOfTiersError, PFCParsingError
from praat	import praat
from praat import scripts as praatScripts
import treetagger.tags
# GUI
from editors   import *
from dialogs import ExitDialog
from metasearch import *
from messages import *
from listbox import *
from preferences import PrefsPanel
from menu	 import PFCMenu
import behaviours


# constants
ORTH     = u'Texte'
SCHWA	 = u'Schwa'
LIAISON  = u'Liaison'
LISTE	 = u'Liste de mots'
ENQUETES = u'Enquêtes'


class PFCBrowser(wx.Frame):
	def __init__(self, parent, id, title, version):
		global PAC

		self.version = version			
		self.metadata = { 'choices': { }, 'values': { }, 'extra': { } } # query metadata
		self.currentMetadata = None # metadata which are currently displayed in the browse panel
		self.tasks = [u'guidé', 'libre', 'texte']	
		self.SetPrefs()
		# store the child Popen instance running Praat when playing a sound file
		self.praatInstance = None

		self.CheckPraat()
		if not os.path.isdir(self.root):
			behaviours.OnOpen(self)
		
		try:
			self.SetCorpus()
		except PFCParsingError, f:
			alert( u"Le fichier '%s' n'a pas pu être chargé. Assurez-vous de ce qu'il soit bien au format Unicode." % f, "Erreur fatale")
			sys.exit()
		# add non-standard tasks
		extraTasks = { 'gx': u'guidé complémentaire', 'lx': u'libre complémentaire', 
						    'tx': u'texte complémentaire' }

		for x in extraTasks:
			if self.corpus.hasTask(x):
				self.tasks.append(extraTasks[x])
		# test for PAC
		if os.path.exists(os.path.join(self.root, 'pac.txt')):
			self.tasks = ['formel', 'informel', 'texte']
			self.pac = True
		else:
			self.pac = False
			
		self.SetInterface(title)
		
		if self.prefs['update']['check'] == 'True':
			behaviours.CheckUpdate()
		

		
	def __getitem__(self, survey):
		'return survey object.'
		return self.corpus[survey]

#-------------------------------------------------------------------------------------
	def SetInterface(self, title):
		'Set up all GUI stuff.'
		wx.Frame.__init__(self, None, -1, title, (100, 50), (950, 690),
						wx.DEFAULT_FRAME_STYLE ^ (wx.RESIZE_BORDER | wx.MAXIMIZE_BOX))
		

		self.AttachUIToolBar()		
		self.SetFonts()
		
		self.CreateMenu()
		self.CreatePanels()
		
		self.SetBrowsePanel()	
		self.SetQueryPanel()
		self.SetBindings()	
		self.SetListPanel(self.list)
		
		self.Bind(wx.EVT_CLOSE, self.OnQuit)




	def AttachUIToolBar(self):
		"Attach toolbar and set icons (get rid of icons in the future)"
		
		self.icons = { }
		img = os.path.abspath('../Icons')

		praatIcon   = os.path.join(img, 'praat.png')
		folderIcon  = os.path.join(img, 'folder.png')
		playIcon    = os.path.join(img, 'play.png')
		pauseIcon   = os.path.join(img, 'pause.png')
		editIcon    = os.path.join(img, 'edit.png')
		cancelIcon  = os.path.join(img, 'cancel.png')
		speakerIcon = os.path.join(img, 'speaker.png')
		rootIcon    = os.path.join(img, 'tree_root.png') 
		textIcon    = os.path.join(img, 'tree_text.png')
		saveIcon    = os.path.join(img, 'filesave.png')
		helpIcon    = os.path.join(img, 'help.png')
		collectionIcon = os.path.join(img, 'collection.png')

		self.toolbar = self.CreateToolBar()
		
#		ID_PRAAT = wx.NewId()
#		self.toolbar.AddSimpleTool(ID_PRAAT, wx.Bitmap(praatIcon), shortHelpString = u"Ouvrir la sélection dans Praat",
#								longHelpString = u"Ouvrir la sélection dans Praat")
		
		#self.toolbar.AddSeparator()
		
		ID_EDIT = wx.NewId()
		self.toolbar.AddSimpleTool(ID_EDIT, wx.Bitmap(editIcon), shortHelpString = u"Editer la sélection",
														longHelpString = u"Editer l'intervalle TextGrid")


		#callPraat = lambda e: self.OnOpenInPraat()
		callEdit = lambda e: behaviours.OnEditInterval(self)
		#wx.EVT_MENU(self, ID_PRAAT, callPraat)
		wx.EVT_MENU(self, ID_EDIT, callEdit)
		
		#self.toolbar.AddLabelTool(4, '', wx.Bitmap(collectionIcon))
		self.toolbar.Realize()		
		self.toolbar.Show()		
				
		
		icons = [
			('praat', praatIcon), 
			('folder', folderIcon), 
			('play', playIcon),
			('pause', pauseIcon), 
			('root', rootIcon),
			('text', textIcon), 
			('save', saveIcon), 
			('cancel', cancelIcon),
			('speaker', speakerIcon), 
			('help', helpIcon),
			('collection', collectionIcon) 
		]

		for key, icon in icons:
			self.icons[key] = wx.Image(icon, wx.BITMAP_TYPE_ANY).ConvertToBitmap()


	def SetFonts(self):
		fSize = 10
		if sys.platform.startswith('darwin'):
			fSize = 12
		self.monoFont = wx.Font(fSize, wx.MODERN, wx.NORMAL, wx.NORMAL)

	
#----------------------------------------------------------------------------#
	def CreateMenu(self):
		self.menu = PFCMenu(self, self.prefsdir)

		
	def CreatePanels(self):
		'Create and set up all tabs.'
		self.query  = { 'title': u"Recherche" } # qery panel
		self.list  = { 'title': LISTE	}		# store info about word list panel
		self.brows = { 'title': ENQUETES }	# store info about browsing panel

		self.notebook = wx.Notebook(self, -1, wx.DefaultPosition, 
				wx.DefaultSize, wx.NB_TOP)
		for tab in [ self.brows, self.query, self.list ]:
			tab['win'] =  wx.Panel(self.notebook,-1)
			win = tab['win']
			title = tab['title']				
			self.notebook.AddPage(win, title)

	
	def SetQueryPanel(self):
		'Set main Query panel'
		tab = self.query
		tab['res'] = [ ] # store query results
		tab['search'] = { } # store metadata search fields
		win = tab['win']
		# tcQuery: SearchCtrl used to be a simple TextCtrl
		tab['tcQuery'] = wx.SearchCtrl(win, -1, "", pos=(10, 50), style=wx.TE_PROCESS_ENTER)
		tab['tcQuery'].SetDescriptiveText("Motif")
		tab['tcQuery'].ShowCancelButton(True)
		wx.CallAfter(tab['tcQuery'].SetInsertionPoint, 0) 	
		self.Bind(wx.EVT_SEARCHCTRL_CANCEL_BTN, self.OnCancelSearchButton, tab['tcQuery'])	
		runTip = wx.ToolTip(u'Lancer la requête')
		p = 150
		if sys.platform == 'darwin':
                        p = 130
		tab['btnRun']  = wx.Button(win, -1, 'Rechercher', pos = (p, 50))
		tab['btnRun'].SetToolTip(runTip)

		width = self.AttachUIResultListBox(tab)
		tab['stNbrTkn'] = wx.StaticText(win, -1, '', pos=(12,80), size=(width,-1))
	
		x, y = (570, 110)
		height = 185
		self.AttachUISurveys(tab, (x, y))
		self.AttachUITasks(tab, (x, y+90))
		self.AttachUISpeakers(tab, (x, y+height))
		w, z = (40, 435)
		self.AttachUIPraatButton(tab, (w, z))
		self.AttachUIPauseButton(tab, (w+60, z))
		self.AttachUICollSaveButtons(tab, (w+60, z+50))
		self.AttachUIMetadata(tab, (710, y))	
		self.AttachUIMetaSearchBoxes(tab, (180, 420), (570, 420))
		self.AttachUIGramSearch(tab, (570,10))
		process = lambda e: self.RunQuery(tab)
		queryTip = wx.ToolTip(u'Saisir le motif %s à rechercher' % tab['title'].lower())
		tab['tcQuery'].Bind(wx.EVT_TEXT_ENTER, process)
		tab['tcQuery'].SetToolTip(queryTip)

		if self.pac:
			theChoices = ["Texte", "R liaison"]
		else:
			theChoices = ["Texte", "Schwa", "Liaison"] 
		self.tier = wx.Choice(win, -1, (300, 50), choices = theChoices)
		self.Bind(wx.EVT_CHOICE, None, self.tier)	
		
	def OnCancelSearchButton(self, e = None):	
		self.query['tcQuery'].ChangeValue('')
		
	def RefreshResultListBox(self, tab):
		#FIXME: The refresh is buggy when the HtmlListBox is cleared, so we destroy
		# and recreate it for each query. Is it because the length of the list is not updated?
		tab['lbRes'].Destroy()
		self.AttachUIResultListBox(tab)
		self.SetBindings() # update bindings
				
	def SetListPanel(self, tab):
		'Set controls specific to list panel.'
		win = tab['win']		
		tab['search'] = { } # store metadata search fields
		
		self.AttachUIWordListChooser(tab)		
		# if the corpus is empty, don't set the list
		try:
			self.AttachUIWordList(tab)
		except IndexError:
			pass

		x, y = (300, 60)
		height = 90
		
		self.AttachUISurveys(tab, (x, y))
		self.AttachUISpeakers(tab, (x, y+height), (120, 190))
		self.AttachUIPraatButton(tab, (60, 380))
		self.AttachUIPauseButton(tab, (110, 380))
		self.AttachUIMetadata(tab, (460, 60))
		
		# Overwrite OnPlayMatch
		playList = lambda e: self.OnPlayList(tab)
		openList = lambda e: self.OnOpenInPraatList()
		
		self.Bind(wx.EVT_BUTTON, playList, tab['btnPlay'])
		self.Bind(wx.EVT_BUTTON, openList, tab['btnPraat'])

		if not self.pac:
			enableExtra = False
			for survey in self.corpus.keys():
				if self[survey].hasTask("mx"):
					enableExtra = True
					break
			if not enableExtra:
				tab['rbChooser'].DisableTask('mx')
			
		self.AttachUIMetaSearchBoxes(tab, (180, 370), (570, 370))
		tab['btnFilter'] = wx.Button(win, -1, u"Filtrer", pos = (750, 340))
		tip = wx.ToolTip(u"Filtrer les locuteurs à partir des critères de recherche")
		tab['btnFilter'].SetToolTip(tip)
		filter = lambda e: self.OnFilterSpeakers(self.list)
		self.Bind(wx.EVT_BUTTON, filter, tab['btnFilter'])
		
	def OnFilterSpeakers(self, tab):
		"Filter speakers in list tab, according to metadata search criteria."
		tab['search']['speakers'].sendSpeakerMetaSearch()
		task = tab['rbChooser'].selected
		meta = self.metadata	
		
		# first make sure all speakers are selected
		self.OnCheckAllSpeakers(tab)
		
		try:
			selectedSpeakers = self.GetSelectedSpeakers(tab)
		except KeyError:
			alert(u"Sélectionnez une ou plusieurs enquête(s).",
					'Erreur') ; return
		
		logfile = os.path.join(self.prefsdir, 'missing_metadata.txt')
		filteredSpeakers = filterSpeakers(self, selectedSpeakers, self.metadata, logfile)
	
		# filer task metadata
		unfilteredSpeakers = filteredSpeakers.copy()
		filteredSpeakers = { }
		for srv in unfilteredSpeakers:
			for spkr in unfilteredSpeakers[srv]:
				tmeta = self[srv][spkr][task].meta
		
				if meta['extra']['TaskDesc']:
					tags = meta['extra']['TaskDesc'].split(';')
		
					if meta['choices']['TaskDesc'] == True:
						if not 'Description' in tmeta:
							continue
						found = False
						for tag in tags:
							if tmeta['Description'].find(tag) > -1:
								found = True
						if not found:
							continue
							
					elif 'Description' in tmeta:
						found = False
						for tag in tags:
							if tmeta['Description'].find(tag) > -1:
								found = True
						if found:
							continue
					
				try:
					filteredSpeakers[srv].append(spkr)
				except KeyError:
					filteredSpeakers[srv] = [spkr]
	
		unselect = [ ]
		for srv in selectedSpeakers:
			for spkr in selectedSpeakers[srv]:
				if (not srv in filteredSpeakers) or (not spkr in filteredSpeakers[srv]):
					unselect.append(srv+spkr)
		
		self.UnselectSpeakers(tab, unselect)


	def AttachUIWordList(self, tab):
		win = tab['win']
		tab['words'] = self.GetList(tab['rbChooser'].selected)
		
		if tab['words']:
			try:
				tab['wdList'].Destroy()
			except:
				pass
			wx.StaticText(win, -1, 'Liste de mots :', (60, 40)) 
			tab['chkWordList'] = wx.CheckBox(win, -1, pos = (200, 40))

			tab['wdList'] = wx.CheckListBox(win, -1, (60, 60), (200, 300), 
										choices = tab['words']) 
			tab['wdList'].SetFont(self.monoFont)
			
			self.Bind(wx.EVT_CHECKBOX, self.OnCheckWordList, tab['chkWordList'])

	def ResetUIWordList(self, tab):
		"Reset word list when list chooser buttons are switched."
		win = tab['win']
		tab['wdList'].Destroy()

		# TODO: make something better
		try:
			tab['words'] = self.GetList(tab['rbChooser'].selected)
		except IndexError:
			alert(u"Placez l'enquête avec une liste complémentaire dans un corpus isolé", u"Liste complémentaire")
		tab['wdList'] = wx.CheckListBox(win, -1, (60, 60), (200, 300), 
									choices = tab['words']) 
		tab['wdList'].SetFont(self.monoFont)
	
	
	def GetList(self, lst = 'm'):
		'Select the first speaker from the first survey and set their word list as default.'
		words = [ ]
		try:
			keys = self.corpus.keys()
		except IndexError:
			return
		keys.sort()
		srv  = keys[0]
		keys = self.corpus[srv].keys()
		keys.sort()
		i = 0
		# If the first speaker has no list, loop until one is found.
		while True:
			try:
				spkr = keys[i]
				words = self.corpus[srv][spkr][lst].textgrid.getWords()
				break
			except PFCMissingTaskError:
				i += 1
		return words
				
		
				
	def SetBindings(self):
		"New binding method which uses lambda expressions."
		tab = self.query
		title = tab['title']
		
		query	   = lambda e: self.RunQuery(tab)
		openInPraat = lambda e: self.OnOpenInPraat()

		self.Bind(wx.EVT_BUTTON, openInPraat, tab['btnPraat'])
		self.Bind(wx.EVT_BUTTON, query, tab['btnRun'])
		#self.Bind(wx.WXK_RETURN, query, id = tab['tcId'])

#===============================================================================
# The AttachUI* methods attach common elements to tabs 
#===============================================================================
	def AttachUISurveys(self, tab, pos = (500, 60)):
		win = tab['win']
		x, y = pos
		
		wx.StaticText(win, -1, pos = (x, y), label = u"Enquêtes :") 	
		tab['chkAllSrvs'] = wx.CheckBox(win, -1, pos = (x+90, y))			
		surveys = self.corpus.keys()
		tab['chkSurv'] = wx.CheckListBox(win, -1, (x, y+20), (120, 65), 
				choices = surveys)
		tab['chkSurv'].SetFont(self.monoFont)
		#tab['chkSurv'].Check(0)
		
		checkSurvey = lambda e: self.OnCheckSurvey(tab)
		checkAll	= lambda e: self.OnCheckAllSurveys(tab)
	
		self.Bind(wx.EVT_CHECKLISTBOX, checkSurvey, tab['chkSurv'])
		self.Bind(wx.EVT_CHECKBOX, checkAll, tab['chkAllSrvs'])		
			
	def AttachUITasks(self, tab, pos = (580, 150)):
		'Attach Task listbox to tab.'
		win = tab['win']
		x, y = pos
		
		wx.StaticText(win, -1, pos = (x, y), label = u"Tâches : ") 
		tab['chkAllTsks'] = wx.CheckBox(win, -1, pos = (x+90, y))
		tab['chkAllTsks'].SetValue(True)
		tab['chkTsk'] = wx.CheckListBox(win, -1, (x, y+20), (120, 70),
				choices = self.tasks)
		for i in range(len(self.tasks)): # select all tasks
			tab['chkTsk'].Check(i, True)
		tab['chkTsk'].SetFont(self.monoFont)	

		f = lambda e: self.OnCheckAllTasks(tab)
		self.Bind(wx.EVT_CHECKBOX, f, tab['chkAllTsks'])		
		
	def AttachUISpeakers(self, tab, pos = (580, 245), size = (120, 95)):
		'Attach Speaker listbox to tab.'
		win = tab['win']
		x, y = pos
		
		wx.StaticText(win, -1, pos = (x, y), label = "Locuteurs :") 	
		tab['chkAllSpkrs'] = wx.CheckBox(win, -1, pos = (x+90, y))
		tab['chkSpkr'] = wx.CheckListBox(win, -1, (x, y+20), size)			
		tab['chkSpkr'].SetFont(self.monoFont)
		
		f = lambda e: self.OnCheckAllSpeakers(tab)
		self.Bind(wx.EVT_CHECKBOX, f, tab['chkAllSpkrs'])
		if tab == self.list:
			self.Bind(wx.EVT_LISTBOX, self.OnListSpeakerSelected, tab['chkSpkr'])

	def AttachUIPraatButton(self, tab, pos):
		'Attach Praat and Play buttons to tab.'
		win = tab['win']
		x, y = pos
		playTip = wx.ToolTip(u"Jouer la sélection")
		praatTip = wx.ToolTip(u"Ouvrir la sélection dans Praat")
		cancelTip = wx.ToolTip(u"Effacer les critères de recherche")
		
		tab['btnPlay'] = wx.BitmapButton(win, -1, self.icons['play'], pos = (x, y))	
		tab['btnPlay'].SetToolTip(playTip)
		tab['btnPraat'] = wx.BitmapButton(win, -1, self.icons['praat'], pos = (x, y+50))		
		tab['btnPraat'].SetToolTip(praatTip)
		tab['btnErase'] = wx.BitmapButton(win, -1, self.icons['cancel'], pos = (x, y+100))
		tab['btnErase'].SetToolTip(cancelTip)
		playList = lambda e: self.OnPlayMatch(e, tab)
		cancel = lambda e: self.OnEraseSearchCriteria(tab)
		self.Bind(wx.EVT_BUTTON, playList, tab['btnPlay'])
		self.Bind(wx.EVT_BUTTON, cancel, tab['btnErase'])
	
	def AttachUIPauseButton(self, tab, pos):
		win = tab['win']
		x, y = pos
		pauseTip = wx.ToolTip(u"Arrêter le son")
		
		tab['btnPause'] = wx.BitmapButton(win, -1, self.icons['pause'], pos = (x,y))
		tab['btnPause'].SetToolTip(pauseTip)
		pause = lambda e: self.OnPauseSound(tab)
		self.Bind(wx.EVT_BUTTON, pause, tab['btnPause'])		
		
		
	def AttachUICollSaveButtons(self, tab, pos):
		'Attach Collection and Pause buttons to tab'
		win = tab['win']
		x, y = pos
		
		collTip = wx.ToolTip(u"Enregistrer la sélection en tant que collection Praat")
		
		tab['btnColl'] = wx.BitmapButton(win, -1, self.icons['collection'], pos = (x, y))
		tab['btnColl'].SetToolTip(collTip)
		writeColl = lambda e: self.OnWriteCollection(tab)
		self.Bind(wx.EVT_BUTTON, writeColl, tab['btnColl'])

		# Attach Print button for results
		tip = wx.ToolTip(u'Sauvegarder les résultats au format CSV')
		tab['btnCSV'] = wx.BitmapButton(tab['win'], -1, self.icons['save'], pos = (x, y+50))
		tab['btnCSV'].SetToolTip(tip)
		csv = lambda evt: self.OnSaveAsCsv(evt, tab)
		self.Bind(wx.EVT_BUTTON, csv, tab['btnCSV'])	
		
		
	def AttachUIMetaSearchBoxes(self, tab, pos, pos2): 
		tab['search']['speakers'] = MetaSearchBox(self, tab, pos, pos2)


	def AttachUIResultListBox(self, tab):
		"Result list box for schwa and liaison panels."
		width = 550 # modified to make sure it fits
		if sys.platform == 'darwin':
			width -= 20
		tab['lbRes'] = ResultListBox(tab, (10, 110), (width, 300))
					
		play = lambda e: self.OnPlayMatch(e, tab)
		meta = lambda event: self.OnSelectListItem(event, tab)
		self.Bind(wx.EVT_LISTBOX_DCLICK, play, tab['lbRes'])

		self.Bind(wx.EVT_LISTBOX, meta, tab['lbRes'])

		tab['lbRes'].SetFont(self.monoFont)
				
		return width

	
	def AttachUIWordListChooser(self, tab):
		"Choose from word lists in list tab."	
		tab['rbChooser'] = WordListChooser(tab, self, self.pac) 

		
	def UpdateUICheckListBox(self, listbox, choices):
		"Update CheckListBox when the interface is refreshed"
		listbox.Clear()
		for i in range(len(choices)):
			listbox.Insert(choices[i], i)

	def AttachUIGramSearch(self, tab, pos):
		"Attach the grammatical search fields."
		tags = ['-----']
		tagsDisabled = False
		try:
			tags = tags + treetagger.tags.gettags(os.path.abspath('../Resources/tagset.txt'))
		except IOError:
			if tab['title'] == 'Liaison':
				alert(u"Le fichier 'tagset.txt' n'existe pas. L'étiquetage grammatical a été désactivé")
			tagsDisabled = True
		x, y = pos
		win = tab['win']
		
		tab['GrBox'] = wx.StaticBox(win, -1, u"Catégories grammaticales :", pos, (360,85))

		y += 25
		x += 10
		wx.StaticText(win, -1, pos = (x+10, y), label = u'mot précédent')
		wx.StaticText(win, -1, pos = (x+130, y), label = u'mot codé')
		wx.StaticText(win, -1, pos = (x+235, y), label = u'mot suivant')
		# word 0 = previous word
		# word 1 = word containing coding
		# word 2 = next word
		y += 25
		sz = (80, 25)
#		tab['GrWd0'] = wx.TextCtrl(win, -1, '', pos=(x+10, y) , size=sz)
#		tab['GrWd1'] = wx.TextCtrl(win, -1, '', pos=(x+120, y) , size=sz)
#		tab['GrWd2'] = wx.TextCtrl(win, -1, '', pos=(x+230, y) , size=sz)

		tab['GrWd0'] = wx.Choice(win, -1, pos=(x+10, y),  size=sz, choices = tags)
		tab['GrWd1'] = wx.Choice(win, -1, pos=(x+120, y), size=sz, choices = tags)
		tab['GrWd2'] = wx.Choice(win, -1, pos=(x+230, y), size=sz, choices = tags)


		# Disabled temporarily for text and schwa + liaison if 'tagset.txt' doesn't exist
		tagfalse = os.path.join(os.path.abspath('.'), 'tag-false')
		if tab['title'] == 'Texte' or tab['title'] == 'Schwa' or tagsDisabled or os.path.exists(tagfalse):
			tab['GrWd0'].Disable()
			tab['GrWd1'].Disable()
			tab['GrWd2'].Disable()		
			
				
	def AttachUIMetadata(self, tab, pos): # moved for listbox
		"Display metadata about speaker and token."
		# md = metadata
		x, y = pos
		win = tab['win']
		tab['meta'] = { }
		
		box = wx.StaticBox(win, -1, "Informations :", pos=(x, y), size=(220, 300))
		x += 20; y += 30
		
		tab['meta']['Survey']  = wx.StaticText(win, -1, pos = (x, y), label = '')
		tab['meta']['Speaker'] = wx.StaticText(win, -1, pos = (x, y+30), label = '')
		tab['meta']['Task']    = wx.StaticText(win, -1, pos = (x, y+60), label = '')
		tab['meta']['Age']     = wx.StaticText(win, -1, pos = (x, y+90), label = '')
		tab['meta']['Sex']     = wx.StaticText(win, -1, pos = (x, y+120), label = '')
		tab['meta']['Job']     = wx.StaticText(win, -1, pos = (x, y+150), label = '')	
		tab['meta']['DescLab'] = wx.StaticText(win, -1, pos = (x, y+180), label = '')
		tab['meta']['Description'] = wx.TextCtrl(win, -1, pos = (x, y+200), size = (190, 60),
			style = wx.TE_MULTILINE | wx.TE_READONLY)	
		tab['meta']['Description'].Hide()

		
	def AttachUITaskMetaEditor(self):
		x, y = (540, 30)
		self.currentMetadata = 'task'
		tab = self.brows
		win = tab['win']
		tab['meta'] = { }
		tab['rubbish'] = [ ] # keep track of widgets which will be destroyed when an item in the tree is selected
		
		tab['rubbish'].append(wx.StaticBox(win, -1, u'Métadonnées de la tâche :', pos = (x, y), size=(300, 400)))
		x += 20; y += 30
		x2 = x + 100

		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Locuteur :'))
		tab['meta']['Speaker'] = wx.StaticText(win, -1, pos = (x2, y), label = '')
		
		y += 30
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Enquête :'))
		tab['meta']['Survey'] = wx.StaticText(win, -1, pos = (x2, y), label = '')

		y += 30
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Tâche :'))
		tab['meta']['Task'] = wx.StaticText(win, -1, pos = (x2, y), label = '')
		
		y += 30		
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Description :'))	
		y += 30
		tab['meta']['Description'] = wx.TextCtrl(win, -1, '', pos=(x, y) , size=(240, 100), style=wx.TE_MULTILINE)		
		y += 130
		tab['btnWriteMeta'] = wx.Button(win, -1, u'Ecrire les métadonnées', pos = (x, y))
		func = lambda e: behaviours.OnWriteTaskMetadata(self)
		self.Bind(wx.EVT_BUTTON, func, tab['btnWriteMeta'])
	
		
	def AttachUISpeakerMetaEditor(self):
		x, y = (540, 30)
		self.currentMetadata = 'speaker'
		tab = self.brows
		win = tab['win']
		tab['meta'] = { }
		tab['rubbish'] = [ ] # keep track of widgets which will be destroyed when an item in the tree is selected
		
		tab['rubbish'].append(wx.StaticBox(win, -1, u'Métadonnées du locuteur :', pos = (x, y), size=(300, 400)))
		x += 20; y += 30
		x2 = x + 100

		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Locuteur :'))
		tab['meta']['Speaker'] = wx.StaticText(win, -1, pos = (x2, y), label = '')
		
		y += 30
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Enquête :'))
		tab['meta']['Survey'] = wx.StaticText(win, -1, pos = (x2, y), label = '')
		
		y += 30		
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Age :'))
		tab['meta']['Age'] = wx.TextCtrl(win, -1, "", 
				pos=(x2, y), style=wx.TE_PROCESS_ENTER)
		
		y += 30
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Sexe :'))
		tab['meta']['Sex'] = wx.Choice(win, -1, (x2, y), 
				choices = ['Femme', 'Homme'])	

		y += 30		
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Occupation :'))		
		tab['meta']['Job'] = wx.TextCtrl(win, -1, "", pos=(x2, y), 
			style=wx.TE_PROCESS_ENTER)	

		y += 30		
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Description :'))	
		y += 30
		tab['meta']['Description'] = wx.TextCtrl(win, -1, '', pos=(x, y) , size=(240, 100), style=wx.TE_MULTILINE)		
		y += 130
		tab['btnWriteMeta'] = wx.Button(win, -1, u'Ecrire les métadonnées', pos = (x, y))
		func = lambda e: behaviours.OnWriteSpeakerMetadata(self)
		self.Bind(wx.EVT_BUTTON, func, tab['btnWriteMeta'])
		
		
	def AttachUISurveyMetaEditor(self):
		x, y = (540, 30)
		self.currentMetadata = 'survey'
		tab = self.brows
		win = tab['win']
		tab['meta'] = { }
		tab['rubbish'] = [ ] # keep track of widgets which will be destroyed when an item in the tree is selected
		
		tab['rubbish'].append(wx.StaticBox(win, -1, u'Métadonnées de l\'enquête :', pos = (x, y), size=(300, 400)))
		x += 20; y += 30
		x2 = x + 100

		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Enquête :'))
		tab['meta']['Survey'] = wx.StaticText(win, -1, pos = (x2, y), label = '')
		
		y += 30
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Date :'))
		tab['meta']['Date'] = wx.TextCtrl(win, -1, "", pos=(x2, y))
		
		y += 30
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Version :'))
		tab['meta']['Version'] = wx.TextCtrl(win, -1, "", pos=(x2, y))

		y += 30
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Description :'))
		y += 30
		tab['meta']['Description'] = wx.TextCtrl(win, -1, '', pos=(x, y) , size=(240, 100), style=wx.TE_MULTILINE)

		y += 130
		tab['btnWriteMeta'] = wx.Button(win, -1, u'Ecrire les métadonnées', pos = (x, y))
		func = lambda e: behaviours.OnWriteSurveyMetadata(self)
		self.Bind(wx.EVT_BUTTON, func, tab['btnWriteMeta'])


	def AttachUICorpusMetaEditor(self):
		x, y = (540, 30)
		self.currentMetadata = 'corpus'
		tab = self.brows
		win = tab['win']
		tab['meta'] = { }
		tab['rubbish'] = [ ] # keep track of widgets which will be destroyed when an item in the tree is selected
		
		tab['rubbish'].append(wx.StaticBox(win, -1, u'Métadonnées du corpus :', pos = (x, y), size=(300, 400)))
		x += 20; y += 30
		x2 = x + 100

		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Nom :'))
		tab['meta']['Name'] = wx.TextCtrl(win, -1, "", pos=(x2, y))
		
		y += 30
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Date :'))
		tab['meta']['Date'] = wx.TextCtrl(win, -1, "", pos=(x2, y))
		
		y += 30
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Version :'))
		tab['meta']['Version'] = wx.TextCtrl(win, -1, "", pos=(x2, y))

		y += 30
		tab['rubbish'].append(wx.StaticText(win, -1, pos = (x, y), label = u'Description :'))
		y += 30
		tab['meta']['Description'] = wx.TextCtrl(win, -1, '', pos=(x, y) , size=(240, 100), style=wx.TE_MULTILINE)

		y += 130
		tab['btnWriteMeta'] = wx.Button(win, -1, u'Ecrire les métadonnées', pos = (x, y))
		func = lambda e: behaviours.OnWriteCorpusMetadata(self)
		self.Bind(wx.EVT_BUTTON, func, tab['btnWriteMeta'])


#===============================================================================
# Methods related to the processing of metadata in the browse panel
#===============================================================================	
	def UpdateBrowseCorpusMetadata(self):
		if self.currentMetadata != 'corpus':
			self.EraseCurrentMetaEditor()
			self.AttachUICorpusMetaEditor()
			self.currentMetadata = 'corpus'
		
		meta = self.brows['meta']
		for key in ['Name', 'Date', 'Version', 'Description']:
			# make sure previous values are deleted
			meta[key].SetValue('')
			try:
				meta[key].SetValue(self.corpus.meta[key])
			except KeyError:
				value = ''
				
	def UpdateBrowseSurveyMetadata(self, srv):
		if self.currentMetadata != 'survey':
			self.EraseCurrentMetaEditor()
			self.AttachUISurveyMetaEditor()
			self.currentMetadata = 'survey'
		
		meta = self.brows['meta']
		meta['Survey'].SetLabel(srv)
		for key in ['Date', 'Version', 'Description']:
			# make sure previous values are deleted
			meta[key].SetValue('')
			try:
				meta[key].SetValue(self[srv].meta[key])
			except KeyError:
				value = ''

	def UpdateBrowseSpeakerMeta(self, srv, spkr):
		"Update speaker metadata"

		if self.currentMetadata != 'speaker':
			self.EraseCurrentMetaEditor()
			self.AttachUISpeakerMetaEditor()
			self.currentMetadata = 'speaker'
			
		data = self[srv.lower()][spkr.lower()].meta
		editor = self.brows['meta']
		
		editor['Speaker'].SetLabel(spkr)
		editor['Survey'].SetLabel(srv)
		
		for tag in ['Age', 'Job', 'Description']:
			# make sure previous values are deleted
			editor[tag].SetValue('')
			
			try:
				value = data[tag]
			except KeyError:
				value = ''

			editor[tag].SetValue(value)

		try:
			if data['Sex'] == 'Homme':
				i = 1
			else:
				i = 0
			editor['Sex'].SetSelection(i)
		except KeyError:
			pass


	def UpdateBrowseTaskMetadata(self, srv, spkr, tsk):
		if self.currentMetadata != 'task':
			self.EraseCurrentMetaEditor()
			self.AttachUITaskMetaEditor()
			self.currentMetadata = 'task'

		data = self[srv][spkr][tsk].meta
		editor = self.brows['meta']
		
		editor['Speaker'].SetLabel(spkr.upper())
		editor['Survey'].SetLabel(srv)
		editor['Task'].SetLabel(longTaskNames[tsk])
		
		for tag in ['Description']:
			try:
				value = data[tag]
			except KeyError:
				value = ''
				
			editor[tag].SetValue(value)
			
			
				
		
	def OnTreeItemSelected(self, e):
		itemID = e.GetItem()
		label = self.tree.GetItemText(itemID)

		if len(label) == 6 and label.isupper(): 
			srv = label[:3]
			spkr = label[3:]
			self.UpdateBrowseSpeakerMeta(srv, spkr)
			
		elif len(label) == 3 and label.islower():
			self.UpdateBrowseSurveyMetadata(label)
			
		elif label.startswith('Corpus'):
			self.UpdateBrowseCorpusMetadata()
		
		else:
			spkrID = self.tree.GetItemParent(itemID)
			spkr   = self.tree.GetItemText(spkrID)
			self.UpdateBrowseTaskMetadata(spkr[:3].lower(), spkr[3:].lower(), 
				reverseLongTaskNames[label]) 
				
				
	def OnListSpeakerSelected(self, e):
		"Display metadata when a speaker is selected"
		label = e.GetString()
		srv = label[:3]
		spkr = label[3:]
		self.ClearMetadata(self.list)

		meta = self.list['meta'] # metadata text controls
		metadata = self[srv][spkr].meta # speaker metadata
		
		meta['Survey'].SetLabel(u'Enquête : %s' % srv)
		meta['Speaker'].SetLabel(u'Locuteur : %s' % spkr.upper())
		meta['Task'].SetLabel(u'Tâche : liste de mots')
		
		for tag, tagfr in [('Age','Age'), ('Sex', 'Sexe'), ('Job', 'Occupation')]:
			try:
				meta[tag].SetLabel(u'%s : %s' % (tagfr, metadata[tag]))
			except:
				pass

		if 'Description' in meta:
			meta['DescLab'].SetLabel(u'Description :')
			meta['Description'].Show()
			if 'Description' in metadata:
				meta['Description'].SetValue(metadata['Description'])
					
		
		
	def EraseCurrentMetaEditor(self):
		"Erase current metadata in the browse panel."

		for widget in self.brows['meta'].keys():
			self.brows['meta'][widget].Destroy()
			
		for widget in self.brows['rubbish']:
			widget.Destroy()
		
		self.brows['btnWriteMeta'].Destroy()


	def OnEraseSearchCriteria(self, tab):
		tab['search']['speakers'].choices['Sex'].SetSelection(0)
		tab['search']['speakers'].data['Age'].SetValue('')
		tab['search']['speakers'].data['Age2'].SetValue('')
		tab['search']['speakers'].data['Job'].SetValue('')
		tab['search']['speakers'].data['Description'].SetValue('')		
		tab['search']['speakers'].extra['SurveyDesc'].SetValue('')		
		tab['search']['speakers'].extra['TaskDesc'].SetValue('')	
		
		if tab is self.list:
			self.OnCheckAllSpeakers(tab)
		
#===============================================================================
# Preference-related methods
#===============================================================================
	def OnPrefs(self, e):
		'Edit preferences.'
		PrefsPanel(self)

	def SetPrefs(self):
		'Set preferences when the application is started.'
		self.prefsdir  = prefs.getPrefsDir()
		self.prefspath = prefs.getPrefsPath()
		
		# create files on disk if needed
		if not os.path.exists(self.prefsdir):
			os.mkdir(self.prefsdir)
			if sys.platform.startswith('win'):
				prefs.winHide(self.prefsdir)		
								
		if not os.path.exists(self.prefspath):
			prefs.writeINI(self.prefspath, prefs.getDefaultINI())
		# load preferences
#		try:
		self.prefs = prefs.readINI(self.prefspath)
		# the root directory is checked on startup: if it exists, it will be 
		# loaded. If it doesn't, the user is asked to provide a path (see SetCorpus)
		# Adapt root to current user
		r = self.prefs['path']['corpusdir'].replace('/', os.sep)
#		except KeyError:
#			os.rename(self.prefspath, self.prefspath + ".corrupted")
#			alert(u"Le fichier préférences est corrompu.\nUne sauvegarde a été effectuée et la plateforme sera réinitialisée au prochain démarrage",
#				"Fichier corrompu")
#			sys.exit(33)

		self.root = os.path.expanduser(r)
		self.prefs['path']['corpusdir'] = self.root
		# path of the serialized corpus
		self.serializedPath = os.path.join(self.prefs['path']['corpusdir'], 'corpus.data')
		
		self.SetPraatPaths()
		
	def SetPraatPaths(self):
		self.praat = praat.getDefaultPath()
		# update praat module for non-standard windows installs
		if not sys.platform.startswith('win'):
                        return
                praatdir = self.prefs['path']['winpraatdir']
                if not self.praat['path'].startswith(praatdir):
                        self.praat['path'] = os.path.join(praatdir, 'praat.exe')
                        self.praat['run'] = self.praat['path']
                        self.praat['pathcmd'] = os.path.join(praatdir, 'praatcon.exe')	
                        self.praat['sendpraat'] = os.path.join(praatdir, 'sendpraat.exe')
                        # reinitialize
                        praat.initialize(self.praat)
				
	def SavePrefs(self):
		prefs.writeINI(self.prefspath, self.prefs)

	
#===============================================================================
# The following methods process data 	
#===============================================================================
	def SetCorpus(self):
		path = ''
		
		if os.path.isfile(os.path.join(self.prefsdir, 'corpus.data')):
			path = os.path.join(self.prefsdir, 'corpus.data')
		elif os.path.isfile(self.serializedPath):
			path = self.serializedPath
			
		if path != '':
			try:
				from pfc import serialize
				
				bar = wx.ProgressDialog(u"Chargement du corpus", u"Veuillez patienter", 1, 
						style = wx.PD_CAN_ABORT
						| wx.PD_APP_MODAL
						| wx.PD_ELAPSED_TIME)

				try:
					self.corpus = serialize.load(path)
				except:
					alert(u"Le fichier 'corpus.data' est corrompu. Le corpus va être réinitialisées.", 
						u"corruption de donnéees")
					os.remove(path)
					self.SetCorpus()	

				finally:
					bar.Update(1)
					bar.Destroy(); return
					
			except ImportError:
				alert(u"Il existe un corpus sérialisé mais le module python est absent"
					+ "de votre installation", "Module pickle absent") 
		
		try:
			self.corpus = Corpus(self.root)
		except NumberOfTiersError, error:
			alert(str(error), "Erreur de tires")
		except OSError:
			self.corpus = { }
			
	def CheckPraat(self):
		'Windows-only (not tested on Vista).'

		writeError = u"Impossible d'écrire le répertoire Praat." \
				+ u"Assurez vous que vous avez les droits nécessaires."
				
		if not sys.platform.startswith('win'):
			return
		
		for name in ['praat', 'praatcon', 'sendpraat']:
			programPath = os.path.join(self.prefs['path']['winpraatdir'], name + '.exe')
			if not os.path.exists(programPath):
				alert(name + u" n'est pas installé sur votre ordinateur. La plateforme va fermer.")
				sys.exit()
			
				
	def OnPlayList(self, tab):
		"Play selected words for any list (default is standard ('m'))."
		task = tab['rbChooser'].selected
		# Get selected items
		selectedWords = self.GetSelectedWords(tab)
		
		selectedItems = [ ]
		for wd in selectedWords:
			nbr = int(wd.split()[0]) - 1 # get index
			selectedItems.append(nbr)
		if not selectedItems:
			alert(u'Sélectionnez un ou plusieurs mot(s).')
			return
		# Get speakers
		try:
			selectedSpeakers = self.GetSelectedSpeakers(tab)
		except KeyError:
			alert(u"Sélectionnez une ou plusieurs enquête(s).",
					'Erreur')
			return

		itemsToPlay = [ ]	
		
		for survey in selectedSpeakers:
			for speaker in selectedSpeakers[survey]:
				try:
					tsk = self[survey][speaker][task]
					tgd = tsk.textgrid
					snd = tsk.wav
					
					times = [ ]
					for item in selectedItems:
						interval = tgd[0][item]
						times.append((interval.xmin, interval.xmax))
								
					itemsToPlay.append((snd, times))
				except PFCMissingTaskError:
					if task == 'm':
						kind = 'standard'
					elif task == 'mx':
						kind = u"complémentaire"
					elif task == 'c':
						kind = u"consonnes"
					elif task == 'v':
						kind = u"voyelles"
					alert(u"Le locuteur %s%s n'a pas de liste de mot %s." % (survey.upper(), speaker.upper(), kind), \
						 u"Tâche manquante")
		
		self.PlayItemsInPraat(itemsToPlay)

		
	def PlayItemsInPraat(self, items):
		'''
		Play items in Praat: 'items' is a list of (soundobject, (begin, end)) tuples.
		'''

		try:		
			self.praatInstance = praat.playFiles(items)
			#TODO: play all items in one script so that it can be killed easily 
		except praat.PraatShellError:
			alert(u"Problème dans l'exécution de Praat. Assurez-vous que le chemin d'accès est correct.")
	
	
	def RunQuery(self, panel):
		self.ClearMetadata(panel)
		coding = panel['tcQuery'].GetValue()
		
		if not coding:
			alert("Saisissez un motif de recherche", "Erreur")
			return

		# Get selected tasks
		userTasks = set()
		for i in range(len(self.tasks)):
			if panel['chkTsk'].IsChecked(i):
				tsk = panel['chkTsk'].GetString(i)
				userTasks.add(reverseShortTaskNames[tsk])
		
		# Get speakers
		try:
			userSpeakers = self.GetSelectedSpeakers(panel)
		except KeyError:
			alert(u"Sélectionnez une ou plusieurs enquête(s).",
					'Erreur')
			return		

		# process metadata
		panel['search']['speakers'].sendSpeakerMetaSearch()
		logfile = os.path.join(self.prefsdir, 'missing_metadata.txt')
		userSpeakers = filterSpeakers(self, userSpeakers, self.metadata, logfile)

		pfcTier = self.GetCurrentTier()
		
		# Get categories
		cats = self.GetCategories(panel)
		##print cats
		
		# append results for each speaker in the speaker list box
		panel['res'] = [ ]
		ptn = None
		if pfcTier == 'texte':
			ptn = compileText(coding)
		elif pfcTier == 'schwa':
			ptn = compileSchwa(coding)
		elif pfcTier == 'liaison':
			ptn = compileLiaison(coding)
		elif pfcTier == 'r liaison':
			ptn = compileRLiaison(coding)
		else:
			print "pb tier : ", pfcTier
			
		# users select whether missing or misformated files should be ignored
		if self.prefs['application']['catchmissing'] == 'True':
			catch = True
		else:
			catch = False
			
		# First get all codings for the selected tasks
		for srv in userSpeakers.keys(): 
			# results are searched in the corpus and not in the 'total' because a user may
			# want to look for a pattern such as 'che****mise****s', whose size is bigger than a
			# match in 'total'. There is no noticeable performance difference.
			
			panel['res'] += self.corpus[srv].getCodings(self, pfcTier, ptn, catch, self.metadata, 
				True, cats, tasks = userTasks, speakers = userSpeakers[srv])
			#TODO: check whether True is still needed above

		if len(panel['res']) == 0:
			alert(u"Aucun codage ne correspond à votre requête !")
			panel['stNbrTkn'].SetLabel("0 occurrence.")
			return
		
		# update panel				
		self.RefreshResultListBox(panel)		
		panel['lbRes'].results = panel['res']
		panel['lbRes'].SetItemCount(len(panel['res']))

		# Write basic info above the results.
		plur = ''
		if len(panel['res']) > 1:
			plur = 's'
			
		foundStr = u"%d occurrence%s trouvée%s" % (len(panel['res']), plur, plur)
		panel['stNbrTkn'].SetLabel(foundStr)


	def GetCategories(self, tab):
		"Return categories selected by the user"
		cats = [ ]
		
		for tag in ('GrWd0', 'GrWd1', 'GrWd2'):
			id = tab[tag].GetCurrentSelection()
			# make sure category is consistent with that of pfc.standard
			if id == 0 or id == wx.NOT_FOUND:
				cat = ''
			else:
				cat = tab[tag].GetString(id)
				 
			cats.append(cat)
			
		if not cats[0] and not cats[1] and not cats[2]:
			return False
			
		return cats
		
	def GetSelectedWords(self, tab):
		selected = [ ]	
		
		for i in range(len(tab['words'])):
			if tab['wdList'].IsChecked(i):
				selected.append(tab['wdList'].GetString(i))
				
		return selected
	
	def GetSelectedSpeakers(self, tab):
		'Return a dictionary (surveys) of lists (selected speakers) in the speaker list box.'
		selected = { }
		
		for i in range(tab['nbrOfSpeakers']):
			if not tab['chkSpkr'].IsChecked(i):
				continue
			id = tab['chkSpkr'].GetString(i)
			srv, spkr = id[:3], id[3:]
			
			if srv in selected:
				selected[srv].append(spkr)
			else:
				selected[srv] = [spkr]
				
		return selected
		
	def UnselectSpeakers(self, tab, speakers):
		'Unselect speakers from a speaker list.'
		for i in range(tab['nbrOfSpeakers']):
			label = tab['chkSpkr'].GetString(i)
			if label in speakers:
				tab['chkSpkr'].Check(i, False)

#----------------------------------------------------------------------------#
	def ResetBrowseTree(self):
		try:
			self.tree.Destroy()
		except AttributeError:
			self
		self.SetBrowseTree()
			
			
	def SetBrowsePanel(self):
		"""Set browsing panel."""
		win = self.brows['win']
		
		tip = wx.ToolTip(u'Ouvrir la tâche sélectionnée dans Praat')
		self.brows['btnPraat'] = wx.BitmapButton(win, -1, self.icons['praat'], pos = (50, 400))
		self.brows['btnPraat'].SetToolTip(tip)
		self.SetBrowseTree()
		
		idAlias = wx.NewId()
		aliasTip = wx.ToolTip(u'Crée un alias vers une enquête dans le corpus')
		self.brows['btnSrvAlias'] = wx.Button(win, idAlias, u'Ajouter une enquête', (120, 400))
		self.brows['btnSrvAlias'].SetToolTip(aliasTip)
		createAlias = lambda e: behaviours.OnAddSurveyToCorpus(self)

		idTrans = wx.NewId()
		tip = wx.ToolTip(u'Convertir les tires TextGrid sélectionnées au format texte brut (TXT)')
		btnTrans = wx.Button(win, idTrans, 'Convertir', (320, 160))
		btnTrans.SetToolTip(tip)
		
		rbOptions = ['Toutes les tires', 'Tire orthographique', 'Tire schwa',
				'Tire liaison']
		self.brows['rbTrans'] = wx.RadioBox(win, -1, u'Transformer en texte brut', 
				(320, 30), wx.DefaultSize, rbOptions, 1, wx.RA_SPECIFY_COLS)  		

		self.Bind(wx.EVT_BUTTON, createAlias, id = idAlias)
		self.Bind(wx.EVT_BUTTON, self.OnTranspraat, id = idTrans)
		
		self.AttachUICorpusMetaEditor() # just to make sure an editor is destroyed
		self.UpdateBrowseCorpusMetadata()

	def OnTreeItem(self, evt):
		try:
			taskID = self.tree.GetSelection()
			spkrID = self.tree.GetItemParent(taskID)
			survID = self.tree.GetItemParent(spkrID)
			
			task = reverseLongTaskNames[ self.tree.GetItemText(taskID) ]
			spkr = self.tree.GetItemText(spkrID)[3:].lower()
			surv = self.tree.GetItemText(survID)
			
			#print task, spkr, surv
			tgd = self.corpus[surv][spkr][task].textgrid
			snd = self.corpus[surv][spkr][task].wav

			try:				
				praat.sendPraatEdit(snd, tgd)
			except praat.PraatShellError:
				alert(u"Problème dans l'exécution de Praat. Assurez-vous que le chemin d'accès est correct.")
		except:
			pass
		

	def SetBrowseTree(self):
		# Set the tree control of the first panel.
		win = self.brows['win']
		self.tree = wx.TreeCtrl(win, -1, pos = (50,30), size = (250, 350)) 
		self.tree.SetFont(self.monoFont)
		
		il = wx.ImageList(16, 16)
		self.rootId   = il.Add(self.icons['root'])
		self.folderId = il.Add(self.icons['folder'])
		self.textId   = il.Add(self.icons['text'])
		self.spkImgId = il.Add(self.icons['speaker']) 
				
		self.tree.AssignImageList(il)
		
		if 'Name' in self.corpus.meta:
			corpusname = "Corpus " + self.corpus.meta['Name']
		else:
			corpusname = "Corpus PFC"
		root = self.tree.AddRoot(corpusname, image=self.rootId)
		
		for srv in self.corpus.keys():
			srvNode = self.tree.AppendItem(root, self.corpus[srv].id, 
					image = self.folderId)
			
			for spkr in self.corpus[srv].keys():
				spkrNode = self.tree.AppendItem(srvNode, str(self.corpus[srv][spkr]), 
						image = self.spkImgId)
				
				for tsk in self.corpus[srv][spkr].keys():
					tskNode = self.tree.AppendItem(spkrNode, longTaskNames[tsk],
							image = self.textId)
							
		self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnTreeItem, self.tree)
		self.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnTreeItemSelected, self.tree)
		self.Bind(wx.EVT_BUTTON, self.OnTreeItem, self.brows['btnPraat'])		


	def GetTreeItem(self):		
		taskID = self.tree.GetSelection()
		spkrID = self.tree.GetItemParent(taskID)
		survID = self.tree.GetItemParent(spkrID)
		
		task = reverseLongTaskNames[ self.tree.GetItemText(taskID) ]
		spkr = self.tree.GetItemText(spkrID)[3:].lower()
		surv = self.tree.GetItemText(survID)
		
		return [surv, spkr, task]

	
	def OnTranspraat(self, e):
		try:
			surv, spkr, task = self.GetTreeItem()
		except:
			alert(u'Vous devez sélectionner un fichier', 'Erreur')
			return
		if task == 'm': # Transpraat not available for word list
			return

		tier = self.brows['rbTrans'].GetSelection()
		txt  = self.corpus[surv][spkr][task].textgrid.transpraat(tier)
		
		TextEditor(self, 'Transpraat : ' + (surv + spkr).upper(), txt)


#-----------------------------------------------------------------------------#
	def OnOpenInPraatList(self):
		try:
			surveys = self.GetSelectedSpeakers(self.list)
		except KeyError:
			alert(u"Vous devez sélectionner un locuteur.")
			return
		items   = self.GetSelectedWords(self.list)
	
		# only the first word for the first speaker is opened
		keys = surveys.keys()
		keys.sort()
		try:
			survey  = keys[0]
		except IndexError:
			alert(u"Sélectionnez un mot."); return
		speaker = surveys[survey][0]
		task	= self[survey][speaker]['m']
		try:
			item = items[0]
		except IndexError:
			alert(u"Vous devez sélectionner un mot.")
			return
		i = int(item.split()[0]) - 1
		xmin = task.textgrid.transcription[i].xmin

		self.OpenInPraat(task, 1, xmin)
						
	def GetCurrentTier(self):
		return self.tier.GetString(self.tier.GetSelection()).lower()
	
	def OnOpenInPraat(self):
		tier = self.GetCurrentTier()
		
		if tier == 'schwa':
			tier = 2
		elif tier == 'liaison':
			tier = 3
		else:
			tier = 1
		
		try:
			selected = self.GetSelectedMatch()
		except IndexError:
			return
		
		if selected:
			self.OpenInPraat(selected.task, tier, selected.xmin)
	
		
	def OpenInPraat(self, task, tier, xmin):
		if self.notebook.GetCurrentPage() is not self.query['win']:
			return
	
		snd = task.wav.path
		tgd = task.textgrid.path
		
		content = praatScripts.openInPraat(tier, snd, tgd, xmin)

		try:
			praat.runScript(content) #runPlugin('play', content)
		except praat.PraatShellError:
			alert(u"Problème dans l'exécution de Praat. Assurez-vous que le chemin d'accès est correct.")


	def OnWriteCollection(self, tab):
		try:
			selected = self.GetSelectedMatch()
		except IndexError:
			return

		dialog = wx.FileDialog (None, u"Enregistrer sous...", style = wx.SAVE|wx.OVERWRITE_PROMPT, 
							    wildcard = "Praat Collection (*.Collection)|*.Collection")  
		if dialog.ShowModal() == wx.ID_OK: 
			path = dialog.GetPath()
		dialog.Destroy() 
		
		if not path.endswith('.Collection'):
			path += '.Collection'
		snd = selected.task.wav.path
		tgd = selected.task.textgrid.path		
		content = praatScripts.writeSoundTextCollection(snd, tgd, selected.xmin, selected.xmax, path)

		try:
			praat.executeScript(content)
		except praat.PraatShellError:
			alert(u"Problème dans l'exécution de Praat. Assurez-vous que le chemin d'accès est correct.")


	def OnPlayMatch(self, e, tab):
		try:
			selected = self.GetSelectedMatch()
			wav = selected.task.wav
		except IndexError:
			return
		try:
			self.praatInstance = praat.play(wav, selected.xmin, selected.xmax)
		except praat.PraatShellError:
			alert(u"Problème dans l'exécution de Praat. Assurez-vous que le chemin d'accès est correct.")
		
	def OnPauseSound(self, tab):
		if self.praatInstance:
			mysubprocess.kill(self.praatInstance.pid)		
			self.praatInstance = None
		
		
	def OnSelectListItem(self, e, tab):		
		'Display metadata on the token and speaker (not yet) in the result list'
		self.ClearMetadata(tab)

		selected = self.GetSelectedMatch()
		meta = tab['meta'] # metadata text controls
		metadata = selected.task.speaker.meta # speaker metadata
		task = shortTaskNames[selected.task.id]
		
		meta['Survey'].SetLabel(u'Enquête : %s' % selected.task.speaker.survey)
		meta['Speaker'].SetLabel(u'Locuteur : %s' % selected.task.speaker.id.upper())
		meta['Task'].SetLabel(u'Tâche : %s' % task)
		
		for tag, tagfr in [('Age','Age'), ('Sex', 'Sexe'), ('Job', 'Occupation')]:
			if tag in metadata:
				value = metadata[tag]
			else:
				value = '?'
			meta[tag].SetLabel(u'%s : %s' % (tagfr, value))

		if 'Description' in meta:
			meta['DescLab'].SetLabel(u'Description :')
			meta['Description'].Show()
			if 'Description' in metadata:
				meta['Description'].SetValue(metadata['Description'])
			
	def ClearMetadata(self, tab):
		for key in tab['meta'].keys():
			if key == 'Description':
				tab['meta'][key].SetValue('')
				tab['meta'][key].Hide()
			else:
				tab['meta'][key].SetLabel('')
		
	def GetSelectedMatch(self):
		'Get selected item in the result listbox (schwa and liaison)'
		panel = self.query
		no = panel['lbRes'].GetSelection()
		if no == -1:
			return None
		return panel['res'][no]		
	
	def OnPrintResults(self, e, tab):
		"Write results to HTML file." # TODO: should be removed, not needed any longer
		html = tab['lbRes'].ToHtml()
		
		if html:
			editor = TextEditor(self, u"Résultats %s" % tab['title'],
					 			wildcard = "Fichier HTML (*.html)|*.html", visible = False, 
					 			defaultFile = "resultats_%s.html" % tab['title'])
			editor.OnSave(None, html)
			editor.OnQuit(None)


	def OnSaveAsCsv(self, e, tab):
		results = tab['lbRes'].ToCsv()
		
		if results:
			editor = TextEditor(self, u"Résultats %s" % tab['title'],
					 			wildcard = "Fichier CSV (*.csv)|*.csv", visible = False, 
					 			defaultFile = "resultats_%s.csv" % tab['title'])
			editor.OnSave(None, results)
			editor.OnQuit(None)
			
#----------------------------------------------------------------------------#
	def OnCheckSurvey(self, tab):
		'''
		update speakers: erase all speakers, and add those from checked
		surveys.
		'''
		surveys = self.corpus.keys()
		listbox = tab['chkSpkr'] 
		listbox.Clear()
		
		index = -1
		surveyChecked = False
		for i in range(len(surveys)):
			if tab['chkSurv'].IsChecked(i):
				surveyChecked = True
				survey = surveys[i]
				for speaker in self.corpus[survey].keys():
					speaker = survey + speaker
					index = listbox.Append(speaker)
					listbox.Check(index, True)
		
		tab['chkAllSpkrs'].SetValue(surveyChecked)	
		tab['nbrOfSpeakers'] = index + 1


	def OnCheckAllSurveys(self, tab):
		surveys = self.corpus.keys()
		
		value = tab['chkAllSrvs'].IsChecked()
		for i in range(len(surveys)):
			tab['chkSurv'].Check(i, value)
						
		self.OnCheckSurvey(tab)	

					
	def OnCheckAllSpeakers(self, tab):
		value = tab['chkAllSpkrs'].IsChecked()
		
		try:
			for i in range(tab['nbrOfSpeakers']):
				tab['chkSpkr'].Check(i, value)
		except KeyError:
			tab['chkSurv'].Check(0, True)
			self.OnCheckSurvey(tab)
			
	def OnCheckAllTasks(self, tab):
		maxTask = 15 # arbitrary maximum number of tasks
		value = tab['chkAllTsks'].IsChecked()
		
		try:
			for i in range(maxTask):
				tab['chkTsk'].Check(i, value)
		except KeyError:
			return

	def OnCheckWordList(self, e = None):
		tab = self.list
		value = tab['chkWordList'].IsChecked()
		
		try:
			for i in range(len(tab['words'])):
				tab['wdList'].Check(i, value)
		except KeyError:
			alert(u"KeyError dans tab['chkWordList'] (ne nous inquiétez pas...).")	


#===============================================================================
# Methods bound to basic menu items
#===============================================================================
		
	def OnReopen(self, e= None):
		behaviours.OnOpen(self)
		# update
		self.SetCorpus()
		# Refresh GUI
		self.ResetBrowseTree()
		
		surveys = self.corpus.keys()
		surveys.sort()
		for tab in [self.query, self.list]:
			self.UpdateUICheckListBox(tab['chkSurv'], surveys)
			#TODO: update tasks when non standard tasks are added
		
		#self.AttachUIListBox(self.list)
		

	def OnQuit(self, e):
		import sys
		sys.exit()
#		quit = False
#		dlg = wx.MessageDialog(None, u'Etes-vous sûr de vouloir quitter la Plateforme PFC ?', 'Fermeture', 
#            wx.YES_NO | wx.NO_DEFAULT | wx.ICON_QUESTION)
#
#		value = dlg.ShowModal()		
#		if value == wx.ID_YES:
#			quit = True
#		
#		if quit:
#			behaviours.QuitPlatform()
	


		
