# -*- coding: utf-8 -*-

"""
This module defines the basic classes of the PFC project as well as 
standard tasks.
"""

#TODO: pfc.py and standard.py should be reworked and given more meaningful names.

# NB: The platform stores metadata and extra files in a 'data' subdirectory in each speaker folder.

import os, sys
#import metadata
from core.xmlwrap import getTagsFromFile
# praat module
import praat
from praat.textgrid import TextGrid, IntervalTier, TextGridUnicodeError
from praat.sound import LongSound
# pfc module
from standard import StandardTextFile, WordList
import wx
from gui.messages import alert, info

# Standard tasks
LISTE = 'm'
LIST1 = 'v' # pac
LIST2 = 'c' # pac
GUIDE = 'g'
FORMAL = 'f' # pac
TEXTE = 't'
LIBRE = 'l'
INFORMAL = 'i' # pac

standardTasks = [LIBRE, GUIDE, LISTE, TEXTE]
PAC = False
#Texte complémentaire : 11amg1tgx.TextGrid / 11amg1twx.wav


shortTaskNames = {
	'm': u'liste',
	't': u'texte',
	'l': u'libre',
	'g': u'guidé',
	'mx': u'liste complémentaire',
	'tx': u'texte complémentaire',
	'lx': u'libre complémentaire',
	'gx': u'guidé complémentaire',
	'c': u'list C',
	'v': u'list V',
	'f': u'formel',
	'i': u'informel'
}

longTaskNames = {
	'm': u'Liste de mots',
	't': u'Texte lu',
	'l': u'Entretien libre',
	'g': u'Entretien guidé',
	'mx': u'Liste complémentaire',
	'tx': u'Texte complémentaire',
	'lx': u'Entretien (libre) complémentaire',
	'gx': u'Entretien guidé complémentaire',
	'c': u'Liste consonnes',
	'v': u'Liste voyelles',
	'f': u'Entretien formel',
	'i': u'Entretien informel'

}

reverseLongTaskNames = {
	u'Liste de mots':'m',
	u'Texte lu': 't',
	u'Entretien libre': 'l',
	u'Entretien guidé': 'g',
	u'Liste complémentaire': 'mx',
	u'Texte complémentaire': 'tx',
	u'Entretien (libre) complémentaire': 'lx',
	u'Entretien guidé complémentaire': 'gx',
	u'Liste consonnes': 'c',
	u'Liste voyelles': 'v',
	u'Entretien formel': 'f',
	u'Entretien informel': 'i'
}

reverseShortTaskNames = {
	u'liste':'m',
	u'texte': 't',
	u'libre': 'l',
	u'guidé': 'g',
	u'liste complémentaire': 'mx',
	u'texte complémentaire': 'tx',
	u'libre complémentaire': 'lx',
	u'guidé complémentaire': 'gx',
	u'list C': 'c',
	u'list V': 'v',
	u'formel': 'f',
	u'informel': 'i',
}

# Standard values
SURVEY_NAME_LENGTH = 3

#----------------------------------------------------------------------------#	
class PFCException(Exception):
	def __init__(self, utf):
		self.txt = utf
		
	def __unicode__(self):
		return self.txt

class PFCSearchError(Exception):
	pass

class PFCMissingTaskError(PFCException):
	pass

class PFCMissingWavError(Exception):
	pass

#----------------------------------------------------------------------------#		
class Corpus:
	def __init__(self, parentDir, gui = True):
		self.path = os.path.expanduser(parentDir)
		self.surveys = { }
		self.meta = self.parseMeta()
		# check whether this is a PAC corpus
		global PAC
		if os.path.exists(os.path.join(self.path, 'pac.txt')):
			PAC = True

		
		self.getSurveys()
		
	def __len__(self):
		return len(self.surveys.keys())
		
	def __iter__(self):
		return iter(self.surveys.keys())
		
	def __getitem__(self, item):
		return self.surveys[item]
		
	def keys(self):
		k = self.surveys.keys()
		k.sort()
				
		return k
		
	def parseMeta(self):
		'Look for file "corpus-metadata.xml" and parse it.'
		path = os.path.join(self.path, 'corpus-metadata.xml')
		
		if os.path.exists(path):
			return getTagsFromFile(path, ['Date', 'Name', 'Version', 'Description'])	
		else:
			return { }
		
	def getSurveys(self):
		surveys = [ ]
		
		# look for valid surveys to display progress bar
		for survey in os.listdir(self.path):
			if (os.path.isdir(os.path.join(self.path, survey)) \
				and SURVEY_NAME_LENGTH <= len(survey) <= SURVEY_NAME_LENGTH + 2) \
				or survey.lower().endswith('.surveyalias'):
				surveys.append(survey)
				
		numberSurveys = len(surveys)
		# FIXME: the win version needs the 'close' button to be pressed if the style is set.
		if sys.platform.startswith('win'):
			bar = wx.ProgressDialog(u"Chargement des enquêtes", u"Veuillez patienter", numberSurveys)
		else:
			bar = wx.ProgressDialog(u"Chargement des enquêtes", u"Veuillez patienter", numberSurveys,
								    style = wx.PD_CAN_ABORT | wx.PD_APP_MODAL | wx.PD_ELAPSED_TIME)
		
		# FIXME: the behaviour is different on win/osx and linux (Ignore behaviour on linux (was i = 0).
		i = 1
		for survey in surveys:
			if survey.lower().endswith('.surveyalias'):
				surveydir = os.path.join(self.path, survey)
				directory = os.path.dirname( _getAliasPath(surveydir) )
				survey, ext = os.path.splitext(survey)			
			else:
				directory = self.path

			self.surveys[survey] = Survey(directory, survey)
			perc = float(i) / float(numberSurveys) * 100
			bar.Update(i, u"Enquête %s (%d" % (survey.upper(), perc) + "%)")
			i += 1		

		bar.Destroy()
		
	def hasTask(self, task):
		"Check that at least one speaker has the task."
		for survey in iter(self):
			if self[survey].hasTask(task):
				return True
		return False
		

#----------------------------------------------------------------------------#		
class Survey:
	def __init__(self, parentDir, dir):
		self.path = os.path.join( os.path.expanduser(parentDir), dir )
		self.id  = dir.lower()
		self.speakers = { }
		self.meta = { }

		self.parse()

			
	def __getitem__(self, id):
		if self.speakers.has_key(id):
			return self.speakers[id]
		else: 
			return None
			
			
	def __iter__(self):
		return iter(self.speakers)
			
			
	def __contains__(self, speaker):
		return speaker in self.speakers.keys()
	
	
	def __len__(self):
		return len(self.speakers.keys())
		
		
	def keys(self):
		k = self.speakers.keys()
		k.sort()	
		
		return k
	
	def isStandard(self):
		"""Check whether all speakers have (at least) the 4 standard tasks"""
		for speaker in iter(self):
			if not self[speaker].isStandard():
				return False
		return True
		
	def hasTask(self, task):
		"Check that at least one speaker has the task."
		for speaker in iter(self):
			if self[speaker].hasTask(task):
				return True
		return False
		
	def parse(self):
		"""Process the directory structure and build an in-memory representation
		of the survey"""
		global PAC
		# Process metadata
		metaFile = self.id + '-metadata.xml'
		metaFile = os.path.join(self.path, metaFile)
		self.meta = getTagsFromFile(metaFile, ['Date', 'Version', 'Description'])
			
		for spkr in os.listdir(self.path):
			speakerDir = os.path.join(self.path, spkr)
			
			if not os.path.isdir(speakerDir) or len(spkr) != 6:
				continue
			
			speakerID = spkr[-3:]			

			# if a metadata file exists (e.g. 11atg1.xml), parse it.
			metaFile = self.id + speakerID.lower() + '-metadata.xml'
			metaFile = os.path.join(speakerDir, 'data', 'metadata', metaFile)
			meta = getTagsFromFile(metaFile, ['Age', 'Survey', 'Sex', 'Job', 'Description'])
			self.add(speakerID, meta)

			# Look for TextGrids and try to find corresponding Wav file. Raise an exception if
			# the file is not found
			speakerFiles = os.listdir(speakerDir)
			
			for tgd in speakerFiles:
				if tgd.startswith('.') or tgd.endswith('~'):
					continue
				
				# here begins threading
				base, ext = os.path.splitext(tgd)

				if ext.lower() == '.textgrid' or ext.lower() == '.textalias':
					# try to find wav file
					if base[-1] == 'x':
						bs = base[:-2].lower()
						wav  = bs + 'wx.wav'
						flac = bs + 'wx.flac'
						soundAlias = bs + 'wx.soundalias'
						metaFile = bs + 'x-metadata.xml'
					else:
						bs = base[:-1].lower()
						wav  = bs + 'w.wav'
						flac = bs + 'w.flac'
						soundAlias = bs + 'w.soundalias'
						metaFile = bs + '-metadata.xml'
						tagFile = bs + '-tag.IntervalTier'
					
					# find task metadata
					metaFile = os.path.join(speakerDir, 'data', 'metadata', metaFile)
					meta = getTagsFromFile(metaFile, ['Description'])
					
					# find grammatical tagging
					tagFile = os.path.join(speakerDir, 'data', 'tagging', tagFile)

					soundFound = False
					for file in speakerFiles:
						if file.lower() == flac or file.lower() == wav or file.lower() == soundAlias:
							wav = os.path.join(speakerDir, file)
							tgd = os.path.join(speakerDir, tgd)
							if base[-1] == 'x':
								tsk = base[-3] + 'x'
							else:
								tsk = base[-2]

							# tagging
							if os.path.isfile(tagFile):
								tagtier = IntervalTier()
								tagtier.readFromFile(tagFile)
							else: tagtier = None
							
							self[speakerID].add(tsk, tgd, wav, meta, tagtier)
							
							soundFound = True
					if not soundFound:
						alert(u"Le fichier\n%s\nn'a pas été trouvé:" % \
							  os.path.join(speakerDir, wav), "Fichier son manquant")
						
	
	def add(self, speakerID, meta = { }):
		"""Add a speaker"""
		self.speakers[ speakerID ] = Speaker(speakerID, self, meta)


	def getCodings(self, browser, method, ptn, catch, meta, usemeta, cats, **kwargs):
		"""
		Get text matches or 'schwa' or 'liaison' codings for each speaker in 'speakers' 
		(default all) for each task in 'args'
		"""
		match = [ ]

		if not kwargs.has_key('tasks') or len(kwargs['tasks']) == 0:
			kwargs['tasks'] = set([LIBRE, GUIDE, TEXTE])
			
		if not kwargs.has_key('speakers'):
			kwargs['speakers'] = iter(self)

		#TODO: group the if/else into a cleverer solution
		tmeta = { }		
		if method[0].lower() == 's':
			for spkrID in kwargs['speakers']:
				for t in kwargs['tasks']:
					# filter out tasks on the basis of metadata
					# FIXME: missing tasks are caught elsewhere so we just ignore
					# missing metadata
					try:
						tmeta = browser[self.id][spkrID][t].meta
					except PFCMissingTaskError:
						pass
					if usemeta and meta['extra']['TaskDesc']:
						tags = meta['extra']['TaskDesc'].split(';')

 						if meta['choices']['TaskDesc'] == True:
							if not 'Description' in tmeta:
								continue
							found = False
							for tag in tags:
								if tmeta['Description'].find(tag) > -1:
									found = True
							if not found:
								continue
								
						elif 'Description' in tmeta:
							found = False
							for tag in tags:
								if tmeta['Description'].find(tag) > -1:
									found = True
							if found:
								continue
								
					# if the file has not been filtered out, process it
					try:
						map(lambda x: match.append(x), self[spkrID][t].textgrid.getSchwas(ptn))
					except PFCMissingTaskError, error:
						if catch:
							alert(unicode(error), u"Tâche problématique ou manquante")
						
		elif method[0].lower() == 'l':			
			for spkrID in kwargs['speakers']:
				for t in kwargs['tasks']:
					# filter out tasks on the basis of metadata
					try:
						tmeta = browser[self.id][spkrID][t].meta
					except PFCMissingTaskError:
						pass
					
					if usemeta and meta['extra']['TaskDesc']:
						tags = meta['extra']['TaskDesc'].split(';')

 						if meta['choices']['TaskDesc'] == True:
							if not 'Description' in tmeta:
								continue
							found = False
							for tag in tags:
								if tmeta['Description'].find(tag) > -1:
									found = True
							if not found:
								continue
								
						elif 'Description' in tmeta:
							found = False
							for tag in tags:
								if tmeta['Description'].find(tag) > -1:
									found = True
							if found:
								continue								
					# if the file has not been filtered out, process it

					try:
						map(lambda x: match.append(x), self[spkrID][t].textgrid.getLiaisons(ptn))
					except PFCMissingTaskError, error:
						if catch:
							alert(str(error), u"Tâche problématique ou manquante")

########
		elif method[0].lower() == 't':		
			for spkrID in kwargs['speakers']:
				for t in kwargs['tasks']:
					# filter out tasks on the basis of metadata
					try:
						tmeta = browser[self.id][spkrID][t].meta
					except PFCMissingTaskError:
						pass
									
					if usemeta and meta['extra']['TaskDesc']:
						tags = meta['extra']['TaskDesc'].split(';')

 						if meta['choices']['TaskDesc'] == True:
							if not 'Description' in tmeta:
								continue
							found = False
							for tag in tags:
								if tmeta['Description'].find(tag) > -1:
									found = True
							if not found:
								continue
								
						elif 'Description' in tmeta:
							found = False
							for tag in tags:
								if tmeta['Description'].find(tag) > -1:
									found = True
							if found:
								continue								
					# if the file has not been filtered out, process it

					try:
						map(lambda x: match.append(x), self[spkrID][t].textgrid.getTextMatches(ptn))
					except PFCMissingTaskError, error:
						if catch:
							alert(str(error), u"Tﾉche problﾎmatique ou manquante")

		elif method[0].lower() == 'r':		
			for spkrID in kwargs['speakers']:
				for t in kwargs['tasks']:
					# filter out tasks on the basis of metadata
					try:
						tmeta = browser[self.id][spkrID][t].meta
					except PFCMissingTaskError:
						pass
									
					if usemeta and meta['extra']['TaskDesc']:
						tags = meta['extra']['TaskDesc'].split(';')

 						if meta['choices']['TaskDesc'] == True:
							if not 'Description' in tmeta:
								continue
							found = False
							for tag in tags:
								if tmeta['Description'].find(tag) > -1:
									found = True
							if not found:
								continue
								
						elif 'Description' in tmeta:
							found = False
							for tag in tags:
								if tmeta['Description'].find(tag) > -1:
									found = True
							if found:
								continue								
					# if the file has not been filtered out, process it

					try:
						map(lambda x: match.append(x), self[spkrID][t].textgrid.getRLiaisons(ptn))
					except PFCMissingTaskError, error:
						if catch:
							alert(str(error), u"T?che probl?matique ou manquante")

#######

		else:
			raise PFCSearchError, "Problem in pfc.pfc.getCodings(): unhandled task."
		
		# Do not look for categories if meta is not used (i.e. when looking for all codings)
		if not usemeta or not cats:
			return match

		# filter grammatical codings
		newmatch = [ ]
		for m in match:
			mcats = m.getCategories()
			
			if mcats == cats:
				newmatch.append(m)
			else:
				app = True
				
				for i in range(len(cats)):
					if mcats[i]:
						if not mcats[i].startswith(cats[i]):
							app = False
				if app:
					newmatch.append(m)	
				
		return newmatch
		
		
#----------------------------------------------------------------------------#		
class Speaker:
	def __init__(self, id, survey, meta = { }):
		self.id    = id
		self.survey = survey.id
		self.tasks = { }
		self.path = os.path.join(survey.path, survey.id + id)
		self.meta = meta
		
		
	def __str__(self):
		return (self.survey + self.id).upper()
				
	def __iter__(self):
		return iter(self.tasks.keys())

	def __getitem__(self, task):
		try:
			return self.tasks[task]
		except KeyError:
			raise PFCMissingTaskError, u"La tâche '%s' du dossier %s n'est pas valide." \
					% (longTaskNames[task], self.path)

	def __contains__(self, task):
		return task in self.tasks.keys()
			
	def __len__(self):
		return len(self.tasks.keys())
			
			
	def __iter__(self):
		return iter(self.tasks)
			
	def keys(self):
		k = self.tasks.keys()
		k.sort()
		
		return k

	def add(self, task, textgrid, wav, meta = { }, tagtier = None):
		self.tasks[task] = Task(task, self, textgrid, wav, meta)
		
		if tagtier:
			self.tasks[task].textgrid.setTags(tagtier)
			self.tasks[task].textgrid.splitCodingTiers()


	def hasTask(self, task):
		return self.tasks.has_key(task)
		
	def isStandard(self):
		global standardTasks
		
		for task in standardTasks:
			if not task in self:
				return False
				
		return True
			
	def play(self, task, begin, end):
		self[task].wav.play(begin,end)
		

	def find(self, pattern, *args):
		"""Get matches for each task in 'args'"""
		match = [ ]
		rgx = re.compile(pattern, re.IGNORECASE)
		
		if args == ():
			args = (LIBRE, GUIDE, TEXTE)

		for t in args:
				map( lambda x: match.append(x), self[t].textgrid.find(rgx) )
			
		return match

		
	def getCodings(self, method, coding, *args):
		"""Get 'schwa' or 'liaison' codings for each task in 'args'"""
		match = [ ]
				
		if args == ():
			args = (LIBRE, GUIDE, TEXTE)

		if method.lower() == 'schwa': 
			for t in args:
				map( lambda x: match.append(x), self[t].textgrid.getSchwas(coding) )

		elif method.lower() == 'liaison':			
			for t in args:
				map( lambda x: match.append(x), self[t].textgrid.getLiaisons(coding) )

		return match


#----------------------------------------------------------------------------#	
class Task:
	def __init__(self, id, speaker, tgdpath, wavpath,  meta = { }):
		self.id = id
		self.wav = ''
		self.textgrid = ''
		self.speaker = speaker
		self.meta = meta
		
		self.setFiles(speaker, tgdpath, wavpath)
		
	def __getitem__(self, tier):
		#TODO: this should be reworked as a simple list...
		""" Should be 'transcription', 'schwa' or 'liaison'"""
		return getattr(self.textgrid, tier)
		
	def setFiles(self, spkr, tgd, wav):
		global GUIDE, LIBRE, TEXTE, FORMAL, INFORMAL, LIST1, LIST2, PAC

		if PAC:
			texts = [FORMAL, INFORMAL, TEXTE]
		else:
			texts = [GUIDE, LIBRE, TEXTE]

		#FIXME: 'id' should be something like 't' (standard text) or 'tx' (complementary text)
		if self.id[0] in texts:
			try:
				if tgd.lower().endswith('.textalias'):
					tgd = _getAliasPath(tgd)
		
				self.textgrid = StandardTextFile(tgd, self.speaker, self, PAC) 
				
			except TextGridUnicodeError, error:
				alert("Le fichier %s n'est pas au format UTF-16" % error.path, u"Problème Unicode")
		# word list and non standard tasks
		elif self.id[0] == LISTE or self.id[0] == LIST1 or self.id[0] == LIST2:
			self.textgrid = WordList(tgd, self.speaker, self)
			#TEMPORARY FIX: hides extra intervals in files which are not properly formatted
			self.textgrid.cleanList()
		#TODO: improve to handle other cases (prosody for instance)	
		else:
			self.textgrid = TextGrid(tgd)
		
		if wav.lower().endswith(".soundalias"):
			wav = _getAliasPath(wav)	
		self.wav = LongSound(wav) 
		

	def isStandard(self):
		return self.id == GUIDE or self.id == LIBRE or self.id == LISTE \
			or self.id == TEXTE


def _getAliasPath(path):
	newpath = ''
	
	f = open(path)
	for line in f.readlines():
		if line.startswith("path"):
			newpath = line.split("=")[1]
	f.close()
	
	return newpath.strip()
#----------------------------------------------------------------------------#		
if __name__ == '__main__':
	douzens = Survey('~/PFC/Corpus', '11a')
	for loc in iter(douzens):
		#for task in iter(douzens[loc]):
		print douzens[loc][GUIDE].path
		
