#!/usr/bin/env python
# -*- coding: utf-8 -*-
if __name__ == '__main__':
	import sys
	sys.path.append('../')

import os
from praat.textgrid import *
import re


class NumberOfTiersError(Exception):
	pass

class PFCParsingError(Exception):
	pass

class BuggyMatchWords(Exception):
	pass
#---------------------------------------------------------------------------------------

CONTEXT_LENGTH = 30
MAX_MATCH_SIZE = 4
ORTHO = 0
SCHWA = 1
LIAIS = 2
# PAC
R_LIAIS = 1

def compileText(coding):
	return re.compile(coding, re.IGNORECASE)

def compileLiaison(coding):
	'Compile the string coding into a liaison pattern.'
	cons = '[bcdfgjklmnpqrstvwxzJRSZ]' 
	coding = coding.replace('*', '[0-4]')
	coding = coding.replace('C', cons)
	
	return re.compile(coding, re.IGNORECASE)

def compileSchwa(coding):
	'Compile the string into a schwa pattern.'
	if coding == '':
		coding = '\d\d\d\d'
	else:
		if coding.startswith('X'):
			coding = '[^e]' + coding[1:]
		coding = coding.replace('*', '\d').replace('#', '[34]')
		
	return re.compile(coding, re.IGNORECASE)	

def compileRLiaison(coding):
	'Compile R liaison (PAC) (regex : "[0-3][1-2][1-2](h|rh)?")'
	
	if coding == '':
		coding = "[0-3][1-2][1-2](h|rh)?"
	
	coding = coding.replace('*', '[0-3]')
	
	return re.compile(coding, re.IGNORECASE)
	

def getWavFromTextGrid(textgrid):
	root = os.path.dirname(textgrid)
	textgrid = os.path.basename(textgrid)
	
	base, ext = os.path.splitext(textgrid)
	wav = base[:-1] + 'w.wav'
	
	return os.path.join(root, wav)

def getTextGridFromWav(wav):
	root = os.path.dirname(wav)
	wav = os.path.basename(wav)
	
	base, ext = os.path.splitext(wav)
	tgd = base[:-1] + 'g.TextGrid'
	
	return os.path.join(root, tgd)

#--------------------------------------------------------------------------------------------.
class PFCMatchSet:
	def __init__(self):
		self.set = [ ]
		
	def add(self, item):
		self.set.add(item)


class PFCTextMatch(TextMatch):
	def __init__(self, textgridObject, tierNbr):
		TextMatch.__init__(self, textgridObject, tierNbr)
		
	
	def setInfo(self, speaker, task):
		"""Set PFC-specific information"""
		self.speaker = speaker
		self.task    = task
		
	def getCategories(self):
		"""Return a triplet (word 0, word 1, word 2) of categories corresponding to the match
		and surrounding words.
		"""
		wds = self._matchWords()
		
		# keep tags only
		for i in range(len(wds)):
			if wds[i]:
				beg = wds[i].find('[') + 1
				end = wds[i].find(']')
				
				# make sure there is a tag in the word
				if beg == -1 or end == -1:
					wds[i] = None
				else:
					wds[i] = wds[i][beg:end]
				
		return wds
	
	def _matchWords(self):
		"""Match coded word and return a triplet of tagged words"""
		#TODO: add checking mechanisms to be sure words match properly
		tier = self.textgrid[self.tier]
		
		if self.tier == 1:
			splitInts = self.textgrid.splitSchwa
			coding = 'schwa'
		elif self.tier == 2:
			splitInts = self.textgrid.splitLiaison
			coding = 'liaison'
		else:
			raise NumberOfTiersError, u"Problème de n° de tire dans matchWords()."
		
		# interval containing the match
		matchInt = splitInts[self.interval]
		# interval containing the tagged word
		catInt = self.textgrid.splitTags[self.interval]
		
		if len(matchInt) != len(catInt):
			print catInt
			print matchInt
			print "catInt: %d / matchInt: %d" % (len(catInt), len(matchInt))
			raise BuggyMatchWords, u"La longueur des intervalles codés et étiquetés diffère"
				 
		
		lst = map(lambda x, y: [x,y], catInt, matchInt)

		beg = end = 0
		for i in range(len(lst)):
			# cat1 should be "je[PRO]"; word1 should be "je1132"
			cat1, word1 = lst[i]
			
			if not word1:
				continue

			# get previous and next word
			if i > 0:
				cat0, word0 = lst[i-1]
			else:
				cat0 = word0 = ''
			if i < len(lst) - 1:
				cat2, word2 = lst[i+1]
			else:
				cat2 = word2 = ''
			
			# get the position of the word in the interval
			# Add 1 to take into account the size of a space char
			if word1.endswith("'") or word2.startswith(">") or end == 0:
				space = 0
			else:
				space = 1
			beg = end + space				
			end = beg + len(word1)
			
			#DEBUG
			if self.match in word1:
				xx = i

			if beg < self.pos < end:
				if (not ' ' in self.match) and (not self.match in word1):
					raise BuggyMatchWords, u"Problème avec la recherche de la catégorie"

				return [cat0, cat1, cat2]
		#DEBUG
		s = " ".join(matchInt[:xx+1])
		print "LEFT", self.left
		print "RIGHT", self.right
		print "<%s>" % s.replace("' ", "'")
		raise BuggyMatchWords, u"La correspondance [%s] n'a pas eu lieu (pos = %d)" \
			% (self.match, self.pos)
	
		
class StandardTextFile(TextGrid):
	"""Three-tier TextGrid."""
	def __init__(self, filename, speaker, task, pac=False):
		try:
			TextGrid.__init__(self, filename)
		except TextGridParsingError:
			raise PFCParsingError, filename
		
		self.task     = task
		self.speaker  = speaker
		self.transcription  = self.tiers[0]
		if pac == False:
			try:
				self.schwa    = self.tiers[1]
				self.liaison  = self.tiers[2]
			except:
				raise NumberOfTiersError, u"Le fichier %s a des tires manquantes." \
						% os.path.basename(filename)
		
		# the tag tier is stored on its own
		self.tagTier = None

	
	def getSchwas(self, ptn):
		"""Return all matched schwas with their context as a list of strings"""
		#ptn = compileSchwa(coding)
		return self.find(ptn, SCHWA)


	def getLiaisons(self, ptn):
		"""Return all matched schwas with their context as a list of strings"""
		#ptn = compileLiaison(coding)
		return self.find(ptn, LIAIS)

	def getTextMatches(self, ptn):
		return self.find(ptn, ORTHO)
	
	def getRLiaisons(self, ptn):
		'''PAC R liaison'''
		return self.find(ptn, R_LIAIS)

	def find(self, rgx, tier = 0):
			"""Return all matched rgx with their context as a list of strings"""
			results = [ ]

			for i in range(len(self.tiers[tier])):
				pos = 0
				theCodings = [ ]

				# the interval is scanned. Every token is added to 'schwaCodings' as a
				# tuple (begin, end). For the pattern '11\d2', the interval 
				# "Le1132 chat se1112 lave." returns [ (2,6), (8,12) ].
				while True:
					try:				
						theCodings.append( rgx.search( self.tiers[tier][i].text[pos:] ).span() )
						# update position from the beginning of the interval
						theCodings[-1] = [ x + pos for x in theCodings[-1] ]
						# get new position in the interval (end of last coding)
						pos = theCodings[-1][-1]					
						
					except AttributeError:
						break
				self.expandResults(results, theCodings, i, tier)
					
			return results
		

	def expandResults(self, res, codings, int, tier = 0):	
		"""Expand and ornate matches from their position and append
		them to 'res'."""
		theInt = self.tiers[tier][int].text

		for coding in codings:
			(begin, end) = coding
			token = PFCTextMatch(self, tier) 
			token.setInterval(int)
			token.setInfo(self.speaker, self.task)
			token.match = theInt[begin:end]
			token.pos = begin
			
			# find left and right contexts in previous and following
			# intervals if necessary.
			if begin < CONTEXT_LENGTH:
				j = int-1
				leftInt = ''
				
				while len(leftInt) < CONTEXT_LENGTH:
					try:
						leftInt = self.tiers[tier][j].text + leftInt
						j -= 1
					except IndexError:
							break
										
				if leftInt[-1] != ' ':
					leftInt += ' '
					
				missLeft = len(leftInt) - (CONTEXT_LENGTH - begin)
				token.left = leftInt[missLeft:] + token.left + theInt[:begin]
				
			else:
				token.left = theInt[ (begin - CONTEXT_LENGTH) : begin ]
			
			if end > len(theInt) - CONTEXT_LENGTH:
				j = int + 1
				rightInt = ''
				
				while len(rightInt) < CONTEXT_LENGTH: 
					try:
						rightInt += self.tiers[tier][j].text
						j += 1
					except IndexError:
						break
				
				if theInt[-1] != ' ':
					theInt += ' '
					
				missRight = CONTEXT_LENGTH - ( len(theInt) - end )
				token.right = theInt[end:] + rightInt[:missRight]
				
			else:
				token.right = theInt[ end : end + CONTEXT_LENGTH ]
				
			# chop string if it's too long:
			lm = len(token.match)
			if lm > MAX_MATCH_SIZE:
				newlength = CONTEXT_LENGTH - ((lm - MAX_MATCH_SIZE) / 2)
				token.left = token.left[-newlength:]
				token.right = token.right[:newlength]
				
			res.append(token)
			
			
	def transpraat(self, tier = 0):
		'''Return a TextGrid (tier) as a text file. By default, return all tiers.'''
		output = [ ]

		if tier == 0:
			tiers = self.tiers
		else:
			tiers = [ self[tier-1] ]
		
		for tier in tiers:
			output += '### TIRE %s ###' % tier.name.upper()
			# if the speaker is the same, append to current paragraph, otherwise,
			# create a new paragraph.
			lastSpeaker = '' 
			for interval in tier:
				txt = interval.text.rstrip()
				sep = txt.find(':')
				
				if txt.startswith(lastSpeaker) and lastSpeaker != '':
					output.append( txt[sep+1:] )
				else:
					output.append('\n\n' + txt)
					if sep != -1:
						lastSpeaker = txt[:sep]
			output.append('\n\n')
		return ''.join(output)
	
	def setTags(self, tier):
		"Add a tier containing grammatical categories."
		self.tagTier = tier
		# presplit tags
		self.splitTags = map(lambda x: x.text.split(), tier)
	
	def isStandard(self):
		# to be completed
		return len(self) == 3
	
	def isTagged(self):
		return self.tagTier != None	
	
	def splitCodingTiers(self):
		self.splitSchwa   = map(lambda x: x.text.split(), self.schwa)
		self.splitLiaison = map(lambda x: x.text.split(), self.liaison)
		
		# post-process: split words like "j'ai", "ouais>"
		for tier in self.splitSchwa, self.splitLiaison:
			for i in range(len(tier)):
				# the interval can be dynamically lengthened, so we need to take that into account
				# so that the last items are processed
				intlen = len(tier[i])
				j = 0
				
				while j < intlen:
					w = tier[i][j].lower()
					if w.endswith('.') or w.endswith(','):
						w = w[:-1]

						
					if w.endswith('>'):
						if tier[i][j] == '>':
							j += 1
							continue
						
						elif tier[i][j] == '?>' or tier[i][j] == '.>' or tier[i][j] == ',>':
							tier[i][j:j+1] = tier[i][j]
							j += 1
							continue
						
						elif tier[i][j] == '>.' or tier[i][j] == '>,':
							j += 1
							continue
						
						if tier[i][j].endswith('.') or tier[i][j].endswith(','):
							pt = -2
						else:
							pt = -1
						tier[i][j:j+1] = [ tier[i][j][:pt], tier[i][j][pt:] ]
						#j += 1 
					
					# trim the last consonant because of schwa codings
					if w.startswith("d'ailleur") or w.startswith("d'accor") or w.startswith("d'abor")\
					   or w.startswith(u"c'est-à-dire") or w.startswith("aujourd'hui") \
					   or w.startswith("quelqu'un") or w.startswith("d'autant"):
						pass
					 
					elif w.startswith("d'aujourd'hui"):
						tier[i][j:j+1] = [tier[i][j][:2], tier[i][j][2:]]
					
					elif "'" in w and not w.startswith('grand') \
						and not '(' in tier[i][j]: # don't split inside comments, e.g. "(c'est beau"
												
						  if w == "?'" or w == ".'" or w == ",'" : # split "?'"
						  	tier[i][j:j+1] = w
						  	j += 1
						  	
						  elif len(w) > 2:
							tier[i][j:j+1] = tier[i][j].split("'")
							tier[i][j] += "'"
							try:
								if not "-" in tier[i][j+1]: 
									j += 1 # skip the next item unless it contains "-"
							except IndexError:
								pass
							
					elif w == 'est-ce' or w.endswith(u"-là") or w.endswith("-ci") or w.endswith("-moi") \
					   or w.endswith("-le") or w.endswith("-la"):
						tier[i][j:j+1] = tier[i][j].split("-")
						tier[i][j] += "-"
						j += 1
					
					intlen = len(tier[i])
					j += 1

					


class WordList(TextGrid):
	def __init__(self, filename, speaker, task):
		TextGrid.__init__(self, filename)
		
		self.task     = task
		self.speaker  = speaker
		self.transcription  = self.tiers[0]
		
		
	def getWords(self):
		'Return word list as a list of strings.'
		words = [ ]
		
		for interval in self.transcription:
			words.append(unicode(interval))
		return words
	
	def cleanList(self):
		'''
		Work-around for word lists which are not properly formatted (i.e which contain extra
		invervals).
		'''
		intervals = [ ]
		
		for interval in self.transcription:
			try:
				if interval.text.lstrip()[0].isdigit():
					intervals.append(interval)
			except IndexError: # empty labels
				pass
		self.transcription.intervals = intervals

	
if __name__ == '__main__':
	tgd = StandardTextFile('../test.TextGrid', 'Ju', 'Coucou')
	
	print tgd.transpraat(2)
				
	