import sys, os
import codecs
import subprocess
from time import sleep

class PraatError(Exception):
	pass

class PraatShellError(Exception):
	pass



def initialize(p = { }):	
	global gPraat
	gPraat = p

	if not p:
		gPraat = getDefaultPath()
	
			
def getDefaultPath():
	'Reasonable guesses for praat path for each platform'
	if sys.platform.startswith('win'):
		p = { 
			'path': 	'C:\\Praat\\praat.exe',
			'run': 		'C:\\Praat\\praat.exe',
			'pathcmd': 	'C:\\Praat\\praatcon.exe',
			'sendpraat': 'C:\\Praat\\sendpraat.exe',
			'prefsdir':	_userExpand('Praat'),
			'prefs':   	_userExpand('Praat\\Preferences.ini')
		}
				
	elif sys.platform == 'darwin':
		p = {
			'path': '/Applications/Praat.app/Contents/MacOS/Praat',
			'run': 'open /Applications/Praat.app',
			'pathcmd': '/Applications/Praat.app/Contents/MacOS/Praat', # -
			'sendpraat': '/usr/bin/sendpraat',
			'startup': _userExpand('.praat-user-startUp'),
			'prefsdir': _userExpand('Library/Preferences/Praat Prefs'),
			'prefs': _userExpand('Library/Preferences/Praat Prefs/Prefs'),
		}
		
	else: # suppose we're using a Linux/UNIX system
		p = {
			'path': '/usr/local/bin/praat',
			'run': '/usr/local/bin/praat',
			'pathcmd': '/usr/local/bin/praat', # -
			'sendpraat': '/usr/local/bin/sendpraat',
			'startup': _userExpand('.praat-user-startUp'),
			'prefsdir': _userExpand('.praat-dir'),
			'prefs': _userExpand('.praat-dir/prefs')
		}
			
	return p
			
def updateWinPath(folder):
	paths = getDefaultPath()
	
	for key in ['path', 'run', 'pathcmd']:
		paths[key] = os.path.join(folder, os.path.basename(paths[key]))
		
	return paths
	
#----------------------------------------------------------------------------#
def play(sound, begin = 0, end = -1):
	
	if not isinstance(begin, str):
		begin = str(begin)
	if not isinstance(end, str):
		end = str(end)	

	script = "Open long sound file... %s\nPlay part... %s %s" \
		% (sound.path, begin, end)			
	return executeScript(script)
	

def playFiles(files):
	# files is a list of [sound, [begin, end]] lists
	script = ""
	
	for sound, times in files:
		for begin, end in times:
			begin = str(begin)
			end = str(end)
			script += "Open long sound file... %s\nPlay part... %s %s\n\n" \
		% (sound.path, begin, end)	
		
	return executeScript(script)
	
def stopSound():
	sendPraat("Stop playing sound")
	 	
	
def sendMessage(command): # FIXME : still works on Unix without '-'?
	"""Send a command to Praat through a pipe."""

	praat = os.popen(gPraat['pathcmd'], 'w')
	praat.write(command)
	praat.flush()
	try: praat.close()
	except: pass

def executeScript(command = ''):
	if command:
		"""Create a temporary script and execute it."""
		script = os.path.join( os.path.expanduser('~'), '.temporary.praat' )
			
		_writeFile(script, command)	

		child = subprocess.Popen([gPraat['pathcmd'], script])
		sleep(0.5)
		os.remove(script)
		
		return child
	
	else:
		system(gPraat['run'])
		return None
		
		


def sendPraat(command):
	'execute a script via sendpraat'
	sendpraat = gPraat['sendpraat']
	
	if command.endswith('.praat'):
		command = '%s praat "execute ""%s"""' % (sendpraat, command)
	else:
		command = '%s praat "%s"' % (sendpraat, command)
	
	system(command)

def sendPraatEdit(longsound, textgrid):
	"""Edit a LongSound and a TextGrid via sendpraat."""
	tgdBase = os.path.splitext( os.path.basename(textgrid.path)  )[0]
	sndBase = os.path.splitext( os.path.basename(longsound.path) )[0]
	
	content = """! Generated by the Praat python module
Open long sound file... %s
Read from file... %s
select LongSound %s
plus TextGrid %s
Edit
""" % (longsound.path, textgrid.path, sndBase, tgdBase)

	runScript(content)



def writeSimplePlugin(name, content):
	dir = os.path.join( gPraat['prefsdir'], 'plugin_' + name )
	script = os.path.join(dir, 'setup.praat')
	
	if os.path.exists(dir):
		_removeDir(dir)
	
	os.mkdir(dir)
	_writeFile(script, content) 
	
def removePlugin(name):
	dir = os.path.join( gPraat['prefsdir'], 'plugin_' + name )
	_removeDir(dir)
	
def runScript(content):
	'Run a script using sendpraat'
	script = os.path.join( os.path.expanduser('~'), '.temporary.praat' )
	_writeFile(script, content)
	sendPraat(script)
	sleep(3.0)
	#os.remove(script)
	
	
def runPlugin(name, content):
	writeSimplePlugin(name, content)
	executeScript()
	sleep(1.0)
	removePlugin(name)
#----------------------------------------------------------------------------#
def _writeFile(file, content):
	f = codecs.open(file, 'w', 'utf-16') 
	f.write(content + "\n")
	f.close()

	
def _userExpand(file):
	return os.path.join(os.path.expanduser('~'), file) 

def _removeDir(path):
	"""Remove a directory recursively"""
	for f in os.listdir(path):
		file = os.path.join(path, f)
		
		if os.path.isdir(file):
			_removeDir(file)
		else:
			os.remove(file)
	
	os.rmdir(path)
	
	
def system(command):
	exe = os.system(command)
	if exe != 0:
		raise PraatShellError
#----------------------------------------------------------------------------#
		
initialize()

#----------------------------------------------------------------------------#		
if __name__ == '__main__':
	from sound import *
	from textgrid import *

	tgd = TextGrid('/Users/Julien/PFC/CorpusTest/11a/11adp1/11adp1lg.TextGrid')
	wav = LongSound('/Users/Julien/PFC/CorpusTest/11a/11adp1/11adp1lw.wav')

	edit(wav, tgd)

