
# Written by Julien Eychenne for the PFC project. 
# See http://www.projet-pfc.net
# October 2006

"""This module offers an abstraction layer to manipulate Praat TextGrids easily."""

if __name__ == '__main__':
	import sys
	sys.path.append('..')
import re, os, sys
import codecs
from core.io import File

# avoid recompiling every time
findInterval     = re.compile('intervals\s+\[\d*\]:').search
findIntervalTier = re.compile('class\s+=\s+\"IntervalTier\"').search
		
findName = re.compile('name\s+=\s+"(.*)"').search
findXmin = re.compile('xmin\s+=\s+(\d+\.?\d*)').search	
findXmax = re.compile('xmax\s+=\s+(\d+\.?\d*)').search
findText = re.compile('text\s+=\s+"(.*)"').search
	

#----------------------------------------------------------------------------#
class TextGridNameError(Exception):
	def __init__(self, value):
		self.value = value

	def __str__(self):
		return repr(self.value)

class TextGridParsingError(Exception):
	pass

class TextGridUnicodeError(Exception):
	def __init__(self, path):
		self.path = path

#----------------------------------------------------------------------------#		
class Interval:
	"""Tier interval."""
	def __init__(self, min = 0, max = -1, txt = ''):
		self.xmin = min
		self.xmax = max
		self.text = txt
		
	
	def __repr__(self):
		return self.text
		
		
	
class Point:
	"""Tier point."""
	def __init__(self, time, mark):
		self.time = time
		self.mark = mark
		

#----------------------------------------------------------------------------#
class Tier:
	"""Generic tier. This should not be instantiated directly. 
	Use IntervalTier or TextTier instead."""
	def __init__(self, name = ''):
		self.xmin = -1
		self.xmax = -1
		self.name = name
		
				
	

#----------------------------------------------------------------------------#
class IntervalTier(Tier):
	"""TextGrid interval tier."""
	
	def __init__(self, name = ''):
		Tier.__init__(self, name)
		self.intervals = [ ]


	def __getitem__(self, interval):
		"""tier[i] --> return the ith Interval object."""
		return self.intervals[interval]
	
	def __setitem__(self, interval, text):
		self.intervals[interval] = text
		
		
	def __len__(self):
		"""Return number of intervals."""
		return len(self.intervals)
		
# avoid this call, append directly		
	def append(self, interval):
		"""Add an interval to a tier."""
		self.intervals.append(interval)
	

	def count(self, token):
		"""Count the number of tokens in the tier."""
		n = 0
		
		for interval in self.intervals:
			n += interval.text.count(token)
		
		return n
		
	def join(self, txt = ''):
		"""Display the intervals as a string."""
		return txt.join(map(lambda x: x.text, self.intervals))
	
	def readFromFile(self, path):
		"Read a tier from an IntervalTier file."
		# assumes that the file is correctly formatted. 
		#TODO: Should merge this method with TextGrid.parse().
		global intPtn, xminPtn, xmaxPtn, textPtn 
		
		self.name = os.path.splitext(os.path.basename(path))[0]
		tier = [ ]
		isHeader = True
		f = codecs.open(path, 'r', 'utf-16')
		
		for ln in f.readlines():
			if findInterval(ln):
				try:
					tier.append(interval)
				except NameError:
					pass		
				isHeader = False
				interval = Interval()
				continue
				
			match = findXmin(ln)
			if match:
				if not isHeader:
					interval.xmin = match.group(1)
				continue
			
			match = findXmax(ln)
			if match:
				if not isHeader:
					interval.xmax = match.group(1)
				continue
			
			match = findText(ln)
			if match:
				interval.text = match.group(1)
		
		tier.append(interval)
		self.intervals = tier
		f.close()

	
	def write(self, path, withheader = True):
		"""Write to IntervalTier. If withheader is false, only write intervals."""
		tier = File(path, 'w')
		intervals = [ ]
		
		header = '''File type = "ooTextFile"
Object class = "IntervalTier"

xmin = %s 
xmax = %s
intervals: size = %d 
''' % (self.xmin, self.xmax, len(self))
		
		for i in range(len(self)):
			interval = "intervals [%d]:\n" % (i+1)
			interval += "\txmin = %s\n" % self.intervals[i].xmin
			interval += "\txmax = %s\n" % self.intervals[i].xmax
			interval += '\ttext = "%s"\n' % self.intervals[i].text
			intervals.append(interval)

		if withheader:
			tier.write(header)
		tier.write('\n'.join(intervals))	
		tier.close()
	
	def getText(self):
		"Return the text of the intervals as a list of strings."
		return map(lambda x: x.text, self.intervals)
	
	def setText(self, lst):
		"Set text of intervals from a list of strings."
		for i in range(len(lst)):
			self.intervals[i].text = lst[i]
		
#----------------------------------------------------------------------------#		
class TextTier(Tier):
	"""Point tier."""
	def __init__(self, name = ''):
		Tier.__init__(self, name)
		self.points = [ ]
		
	def __add__(self, point):
		self.points.append(point)

			

#----------------------------------------------------------------------------#


class TextGrid:
	"""Give access to a TextGrid's content. Only IntervalTiers at the moment."""
	
	def __init__(self, filename):
		self.path = filename
		self.tiers = [ ] # store the logical content of the TextGrid 
		
		if os.path.exists(self.path):
			self.parse()
		else:
			raise TextGridNameError, "The file %s does not exist" % self.path
			
			
	def __getitem__(self, tier):
		"""mytextgrid[tier] --> tier object."""
		return self.tiers[tier]

	def __len__(self):
		"""Return number of tiers."""
		return len(self.tiers)
		
	def __iter__(self):
		return iter(self.tiers)

		
	def read(self):
		"""Read the content of the file. """
		try:
			f = codecs.open(self.path, 'r', 'utf-16')
		except:
			print "Error: could not open " + self.path
			return False
		
		try:
			lines = f.readlines() # store the physical content of the TextGrid	
		except UnicodeError:
			print "Unicode problem:", os.path.basename(self.path)
			raise TextGridUnicodeError, os.path.basename(self.path)

		f.close()
		
		return lines
				
			
	def parse(self):
		"""Parse the TextGrid's content. Currently, only intervaltiers."""

		global intPtn, intTierPtn, namePtn
		global xminPtn, xmaxPtn, textPtn
		
		lines = self.read()
		i = 0
		while i < len(lines):
			# first look for a new tier
			match = findIntervalTier(lines[i])
			if match:		
				self.tiers.append(IntervalTier())
				tier = self.tiers[-1]
				# scan name, xmin and xmax
				match = None
				while not match:
					i = i+1
					match = findName(lines[i])
				tier.name = match.group(1)

				match = None 
				while not match:
					i = i+1
					match = findXmin(lines[i])
				tier.xmin = match.group(1)
				
				match = None
				while not match:
					i = i+1
					match = findXmax(lines[i])
				tier.xmax = match.group(1)
			
				i = i+1; continue
			
			# look for an interval
			match = findInterval(lines[i])
			if match:
				tier.intervals.append(Interval())
				interval = tier.intervals[-1]
				
				match = None
				while not match:
					i = i+1
					match = findXmin(lines[i])
				interval.xmin = match.group(1)
							    
				match = None
				while not match:
					i = i+1
					match = findXmax(lines[i])	
				interval.xmax = match.group(1)
				
				match = None
				while not match:
					i = i+1
					try:
						match = findText(lines[i])
					except IndexError:
						print self.path
				interval.text = match.group(1)
			
				i = i+1; continue
				
			# scan next line
			i = i+1		
	


	def getValue(self, tier, interval, value = 'text'):
		return getattr( self.tiers[tier].intervals[interval], value )


	def write(self, path):
		pass
				
def printline(line):
	print line




#----------------------------------------------------------------------------#
class TextMatch:
	"""Generic match object"""
	def __init__(self, textgridObject, tierNbr):
		self.textgrid = textgridObject
		self.tier     = tierNbr
		
		self.match    = ''
		self.left     = ''
		self.right    = ''
		self.pos = -1 # position of the match
		
		self.xmin = -1
		self.xmax = -1
		self.interval = -1

		
##	def __str__(self):
##		"""Return the match string in its context"""
##		return self.left + self.match + self.right

	def __unicode__(self):
		# bug with encoding when using __str__
		"""Return the match string in its context"""
		return self.left + self.match + self.right
		
		
	def setInterval(self, int):
		"""Initialise the matching interval"""
		self.interval = int
		self.xmin =  self.textgrid.getValue(self.tier, int, 'xmin')
		self.xmax =  self.textgrid.getValue(self.tier, int, 'xmax')



    


#----------------------------------------------------------------------------#
if __name__ == '__main__':
	for i in range(5000):
		tgd = TextGrid('/home/julien/dev/dolmen/test/test-utf.TextGrid')
		s = tgd[1][21].text
	#print
	#print "\n\n".join(map(unicode, tgd.tiers[0].intervals))
	#print len(tgd.tiers[2].intervals)
